/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this file,
 * You can obtain one at http://mozilla.org/MPL/2.0/. */

"use strict";

var Cu = Components.utils;
var Ci = Components.interfaces;

Cu.import("resource://gre/modules/XPCOMUtils.jsm");
Cu.import("resource://gre/modules/Services.jsm");
Cu.import("resource://gre/modules/PageThumbs.jsm");
Cu.import("resource://gre/modules/BackgroundPageThumbs.jsm");
Cu.import("resource://gre/modules/NewTabUtils.jsm");

XPCOMUtils.defineLazyModuleGetter(this, "Rect",
  "resource://gre/modules/Geometry.jsm");
XPCOMUtils.defineLazyModuleGetter(this, "PrivateBrowsingUtils",
  "resource://gre/modules/PrivateBrowsingUtils.jsm");

var {
  links: gLinks,
  allPages: gAllPages,
  linkChecker: gLinkChecker,
  pinnedLinks: gPinnedLinks,
  blockedLinks: gBlockedLinks,
  gridPrefs: gGridPrefs
} = NewTabUtils;

XPCOMUtils.defineLazyGetter(this, "gStringBundle", function() {
  return Services.strings.
    createBundle("chrome://browser/locale/newTab.properties");
});

function newTabString(name, args) {
  let stringName = "newtab." + name;
  if (!args) {
    return gStringBundle.GetStringFromName(stringName);
  }
  return gStringBundle.formatStringFromName(stringName, args, args.length);
}

function inPrivateBrowsingMode() {
  return PrivateBrowsingUtils.isContentWindowPrivate(window);
}

const HTML_NAMESPACE = "http://www.w3.org/1999/xhtml";
const XUL_NAMESPACE = "http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul";

const TILES_EXPLAIN_LINK = "https://support.mozilla.org/kb/how-do-tiles-work-firefox";
const TILES_INTRO_LINK = "https://www.mozilla.org/firefox/tiles/";
const TILES_PRIVACY_LINK = "https://www.mozilla.org/privacy/";

//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\transformations.js"

/**
 * This singleton allows to transform the grid by repositioning a site's node
 * in the DOM and by showing or hiding the node. It additionally provides
 * convenience methods to work with a site's DOM node.
 */
var gTransformation = {
  /**
   * Returns the width of the left and top border of a cell. We need to take it
   * into account when measuring and comparing site and cell positions.
   */
  get _cellBorderWidths() {
    let cstyle = window.getComputedStyle(gGrid.cells[0].node, null);
    let widths = {
      left: parseInt(cstyle.getPropertyValue("border-left-width")),
      top: parseInt(cstyle.getPropertyValue("border-top-width"))
    };

    // Cache this value, overwrite the getter.
    Object.defineProperty(this, "_cellBorderWidths",
                          {value: widths, enumerable: true});

    return widths;
  },

  /**
   * Gets a DOM node's position.
   * @param aNode The DOM node.
   * @return A Rect instance with the position.
   */
  getNodePosition: function(aNode) {
    let {left, top, width, height} = aNode.getBoundingClientRect();
    return new Rect(left + scrollX, top + scrollY, width, height);
  },

  /**
   * Fades a given node from zero to full opacity.
   * @param aNode The node to fade.
   * @param aCallback The callback to call when finished.
   */
  fadeNodeIn: function(aNode, aCallback) {
    this._setNodeOpacity(aNode, 1, function() {
      // Clear the style property.
      aNode.style.opacity = "";

      if (aCallback)
        aCallback();
    });
  },

  /**
   * Fades a given node from full to zero opacity.
   * @param aNode The node to fade.
   * @param aCallback The callback to call when finished.
   */
  fadeNodeOut: function(aNode, aCallback) {
    this._setNodeOpacity(aNode, 0, aCallback);
  },

  /**
   * Fades a given site from zero to full opacity.
   * @param aSite The site to fade.
   * @param aCallback The callback to call when finished.
   */
  showSite: function(aSite, aCallback) {
    this.fadeNodeIn(aSite.node, aCallback);
  },

  /**
   * Fades a given site from full to zero opacity.
   * @param aSite The site to fade.
   * @param aCallback The callback to call when finished.
   */
  hideSite: function(aSite, aCallback) {
    this.fadeNodeOut(aSite.node, aCallback);
  },

  /**
   * Allows to set a site's position.
   * @param aSite The site to re-position.
   * @param aPosition The desired position for the given site.
   */
  setSitePosition: function(aSite, aPosition) {
    let style = aSite.node.style;
    let {top, left} = aPosition;

    style.top = top + "px";
    style.left = left + "px";
  },

  /**
   * Freezes a site in its current position by positioning it absolute.
   * @param aSite The site to freeze.
   */
  freezeSitePosition: function(aSite) {
    if (this._isFrozen(aSite))
      return;

    let style = aSite.node.style;
    let comp = getComputedStyle(aSite.node, null);
    style.width = comp.getPropertyValue("width");
    style.height = comp.getPropertyValue("height");

    aSite.node.setAttribute("frozen", "true");
    this.setSitePosition(aSite, this.getNodePosition(aSite.node));
  },

  /**
   * Unfreezes a site by removing its absolute positioning.
   * @param aSite The site to unfreeze.
   */
  unfreezeSitePosition: function(aSite) {
    if (!this._isFrozen(aSite))
      return;

    let style = aSite.node.style;
    style.left = style.top = style.width = style.height = "";
    aSite.node.removeAttribute("frozen");
  },

  /**
   * Slides the given site to the target node's position.
   * @param aSite The site to move.
   * @param aTarget The slide target.
   * @param aOptions Set of options (see below).
   *        unfreeze - unfreeze the site after sliding
   *        callback - the callback to call when finished
   */
  slideSiteTo: function(aSite, aTarget, aOptions) {
    let currentPosition = this.getNodePosition(aSite.node);
    let targetPosition = this.getNodePosition(aTarget.node)
    let callback = aOptions && aOptions.callback;

    let self = this;

    function finish() {
      if (aOptions && aOptions.unfreeze)
        self.unfreezeSitePosition(aSite);

      if (callback)
        callback();
    }

    // We need to take the width of a cell's border into account.
    targetPosition.left += this._cellBorderWidths.left;
    targetPosition.top += this._cellBorderWidths.top;

    // Nothing to do here if the positions already match.
    if (currentPosition.left == targetPosition.left &&
        currentPosition.top == targetPosition.top) {
      finish();
    } else {
      this.setSitePosition(aSite, targetPosition);
      this._whenTransitionEnded(aSite.node, ["left", "top"], finish);
    }
  },

  /**
   * Rearranges a given array of sites and moves them to their new positions or
   * fades in/out new/removed sites.
   * @param aSites An array of sites to rearrange.
   * @param aOptions Set of options (see below).
   *        unfreeze - unfreeze the site after rearranging
   *        callback - the callback to call when finished
   */
  rearrangeSites: function(aSites, aOptions) {
    let batch = [];
    let cells = gGrid.cells;
    let callback = aOptions && aOptions.callback;
    let unfreeze = aOptions && aOptions.unfreeze;

    aSites.forEach(function(aSite, aIndex) {
      // Do not re-arrange empty cells or the dragged site.
      if (!aSite || aSite == gDrag.draggedSite)
        return;

      batch.push(new Promise(resolve => {
        if (!cells[aIndex]) {
          // The site disappeared from the grid, hide it.
          this.hideSite(aSite, resolve);
        } else if (this._getNodeOpacity(aSite.node) != 1) {
          // The site disappeared before but is now back, show it.
          this.showSite(aSite, resolve);
        } else {
          // The site's position has changed, move it around.
          this._moveSite(aSite, aIndex, {unfreeze: unfreeze, callback: resolve});
        }
      }));
    }, this);

    if (callback) {
      Promise.all(batch).then(callback);
    }
  },

  /**
   * Listens for the 'transitionend' event on a given node and calls the given
   * callback.
   * @param aNode The node that is transitioned.
   * @param aProperties The properties we'll wait to be transitioned.
   * @param aCallback The callback to call when finished.
   */
  _whenTransitionEnded:
    function(aNode, aProperties, aCallback) {

    let props = new Set(aProperties);
    aNode.addEventListener("transitionend", function onEnd(e) {
      if (props.has(e.propertyName)) {
        aNode.removeEventListener("transitionend", onEnd);
        aCallback();
      }
    });
  },

  /**
   * Gets a given node's opacity value.
   * @param aNode The node to get the opacity value from.
   * @return The node's opacity value.
   */
  _getNodeOpacity: function(aNode) {
    let cstyle = window.getComputedStyle(aNode, null);
    return cstyle.getPropertyValue("opacity");
  },

  /**
   * Sets a given node's opacity.
   * @param aNode The node to set the opacity value for.
   * @param aOpacity The opacity value to set.
   * @param aCallback The callback to call when finished.
   */
  _setNodeOpacity:
    function(aNode, aOpacity, aCallback) {

    if (this._getNodeOpacity(aNode) == aOpacity) {
      if (aCallback)
        aCallback();
    } else {
      if (aCallback) {
        this._whenTransitionEnded(aNode, ["opacity"], aCallback);
      }

      aNode.style.opacity = aOpacity;
    }
  },

  /**
   * Moves a site to the cell with the given index.
   * @param aSite The site to move.
   * @param aIndex The target cell's index.
   * @param aOptions Options that are directly passed to slideSiteTo().
   */
  _moveSite: function(aSite, aIndex, aOptions) {
    this.freezeSitePosition(aSite);
    this.slideSiteTo(aSite, gGrid.cells[aIndex], aOptions);
  },

  /**
   * Checks whether a site is currently frozen.
   * @param aSite The site to check.
   * @return Whether the given site is frozen.
   */
  _isFrozen: function(aSite) {
    return aSite.node.hasAttribute("frozen");
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\page.js"

// The amount of time we wait while coalescing updates for hidden pages.
const SCHEDULE_UPDATE_TIMEOUT_MS = 1000;

/**
 * This singleton represents the whole 'New Tab Page' and takes care of
 * initializing all its components.
 */
var gPage = {
  /**
   * Initializes the page.
   */
  init: function() {
    // Add ourselves to the list of pages to receive notifications.
    gAllPages.register(this);

    // Listen for 'unload' to unregister this page.
    addEventListener("unload", this, false);
    
    // Listen for toggle button clicks.
    let button = document.getElementById("newtab-toggle");
    button.addEventListener("click", e => this.toggleEnabled(e));

    // XXX bug 991111 - Not all click events are correctly triggered when
    // listening from xhtml nodes -- in particular middle clicks on sites, so
    // listen from the xul window and filter then delegate
    addEventListener("click", this, false);

    // Check if the new tab feature is enabled.
    let enabled = gAllPages.enabled;
    if (enabled)
      this._init();

    this._updateAttributes(enabled);
  },

  /**
   * Listens for notifications specific to this page.
   */
  observe: function(aSubject, aTopic, aData) {
    if (aTopic == "nsPref:changed") {
      let enabled = gAllPages.enabled;
      this._updateAttributes(enabled);

      // Initialize the whole page if we haven't done that, yet.
      if (enabled) {
        this._init();
      } else {
        gUndoDialog.hide();
      }
    } else if (aTopic == "page-thumbnail:create" && gGrid.ready) {
      for (let site of gGrid.sites) {
        if (site && site.url === aData) {
          site.refreshThumbnail();
        }
      }
    }
  },

  /**
   * Updates the page's grid right away for visible pages. If the page is
   * currently hidden, i.e. in a background tab or in the preloader, then we
   * batch multiple update requests and refresh the grid once after a short
   * delay. Accepts a single parameter the specifies the reason for requesting
   * a page update. The page may decide to delay or prevent a requested updated
   * based on the given reason.
   */
  update(reason = "") {
    // Update immediately if we're visible.
    if (!document.hidden) {
      // Ignore updates where reason=links-changed as those signal that the
      // provider's set of links changed. We don't want to update visible pages
      // in that case, it is ok to wait until the user opens the next tab.
      if (reason != "links-changed" && gGrid.ready) {
        gGrid.refresh();
      }

      return;
    }

    // Bail out if we scheduled before.
    if (this._scheduleUpdateTimeout) {
      return;
    }

    this._scheduleUpdateTimeout = setTimeout(() => {
      // Refresh if the grid is ready.
      if (gGrid.ready) {
        gGrid.refresh();
      }

      this._scheduleUpdateTimeout = null;
    }, SCHEDULE_UPDATE_TIMEOUT_MS);
  },

  /**
   * Internally initializes the page. This runs only when/if the feature
   * is/gets enabled.
   */
  _init: function() {
    if (this._initialized)
      return;

    this._initialized = true;

    // XXX: This comment makes no sense for what it does. There is no HC check,
    // and it changes the button unconditionally to a "play" button, which is
    // wrong for a search action. Commented out for now.
    
    // Set submit button label for when CSS background are disabled (e.g.
    // high contrast mode).
    //document.getElementById("searchSubmit").value =
    //  document.body.getAttribute("dir") == "ltr" ? "\u25B6" : "\u25C0";

    if (document.hidden) {
      addEventListener("visibilitychange", this);
    } else {
      setTimeout(() => this.onPageFirstVisible());
    }

    // Initialize and render the grid.
    gGrid.init();

    // Initialize the drop target shim.
    gDropTargetShim.init();

//@line 137 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\page.js"
  },

  /**
   * Updates the 'page-disabled' attributes of the respective DOM nodes.
   * @param aValue Whether the New Tab Page is enabled or not.
   */
  _updateAttributes: function(aValue) {
    // Set the nodes' states.
    let nodeSelector = "#newtab-grid, #searchContainer";
    for (let node of document.querySelectorAll(nodeSelector)) {
      if (aValue)
        node.removeAttribute("page-disabled");
      else
        node.setAttribute("page-disabled", "true");
    }

    // Enables/disables the control and link elements.
    let inputSelector = ".newtab-control, .newtab-link";
    for (let input of document.querySelectorAll(inputSelector)) {
      if (aValue)
        input.removeAttribute("tabindex");
      else
        input.setAttribute("tabindex", "-1");
    }
  },

  /**
   * Handles unload event
   */
  _handleUnloadEvent: function() {
    gAllPages.unregister(this);
  },

  /**
   * Handles all page events.
   */
  handleEvent: function(aEvent) {
    switch (aEvent.type) {
      case "load":
        this.onPageVisibleAndLoaded();
        break;
      case "unload":
        this._handleUnloadEvent();
        break;
      case "click":
        let {button, target} = aEvent;
        // Go up ancestors until we find a Site or not
        while (target) {
          if (target.hasOwnProperty("_newtabSite")) {
            target._newtabSite.onClick(aEvent);
            break;
          }
          target = target.parentNode;
        }
        break;
      case "dragover":
        if (gDrag.isValid(aEvent) && gDrag.draggedSite)
          aEvent.preventDefault();
        break;
      case "drop":
        if (gDrag.isValid(aEvent) && gDrag.draggedSite) {
          aEvent.preventDefault();
          aEvent.stopPropagation();
        }
        break;
      case "visibilitychange":
        // Cancel any delayed updates for hidden pages now that we're visible.
        if (this._scheduleUpdateTimeout) {
          clearTimeout(this._scheduleUpdateTimeout);
          this._scheduleUpdateTimeout = null;

          // An update was pending so force an update now.
          this.update();
        }

        setTimeout(() => this.onPageFirstVisible());
        removeEventListener("visibilitychange", this);
        break;
    }
  },

  onPageFirstVisible: function() {
    for (let site of gGrid.sites) {
      if (site) {
        // The site may need to modify and/or re-render itself if
        // something changed after newtab was created by preloader.
        // For example, the suggested tile endTime may have passed.
        site.onFirstVisible();
      }
    }

    // save timestamp to compute page life-span delta
    this._firstVisibleTime = Date.now();

    if (document.readyState == "complete") {
      this.onPageVisibleAndLoaded();
    } else {
      addEventListener("load", this);
    }
  },

  onPageVisibleAndLoaded() {
  },

  toggleEnabled: function(aEvent) {
    gAllPages.enabled = !gAllPages.enabled;
    aEvent.stopPropagation();
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\grid.js"

/**
 * This singleton represents the grid that contains all sites.
 */
var gGrid = {
  /**
   * The DOM node of the grid.
   */
  _node: null,
  _gridDefaultContent: null,
  get node() { return this._node; },

  /**
   * The cached DOM fragment for sites.
   */
  _siteFragment: null,

  /**
   * All cells contained in the grid.
   */
  _cells: [],
  get cells() { return this._cells; },

  /**
   * All sites contained in the grid's cells. Sites may be empty.
   */
  get sites() { 
    // return [for (cell of this.cells) cell.site];
    let aSites = [];
    for (let cell of this.cells) {
      aSites.push(cell.site);
    }
    return aSites;
  },

  // Tells whether the grid has already been initialized.
  get ready() { return !!this._ready; },

  // Returns whether the page has finished loading yet.
  get isDocumentLoaded() { return document.readyState == "complete"; },

  /**
   * Initializes the grid.
   * @param aSelector The query selector of the grid.
   */
  init: function() {
    this._node = document.getElementById("newtab-grid");
    this._gridDefaultContent = this._node.lastChild;
    this._createSiteFragment();

    gLinks.populateCache(() => {
      this._refreshGrid();
      this._ready = true;
    });
  },

  /**
   * Creates a new site in the grid.
   * @param aLink The new site's link.
   * @param aCell The cell that will contain the new site.
   * @return The newly created site.
   */
  createSite: function(aLink, aCell) {
    let node = aCell.node;
    node.appendChild(this._siteFragment.cloneNode(true));
    return new Site(node.firstElementChild, aLink);
  },

  /**
   * Handles all grid events.
   */
  handleEvent: function(aEvent) {
    // Any specific events should go here.
  },

  /**
   * Locks the grid to block all pointer events.
   */
  lock: function() {
    this.node.setAttribute("locked", "true");
  },

  /**
   * Unlocks the grid to allow all pointer events.
   */
  unlock: function() {
    this.node.removeAttribute("locked");
  },

  /**
   * Renders the grid.
   */
  refresh() {
    this._refreshGrid();
  },

  /**
   * Renders the grid, including cells and sites.
   */
  _refreshGrid() {
    let row = document.createElementNS(HTML_NAMESPACE, "div");
    row.classList.add("newtab-row");
    let cell = document.createElementNS(HTML_NAMESPACE, "div");
    cell.classList.add("newtab-cell");

    // Clear the grid
    this._node.innerHTML = "";

    // Creates the structure of one row
    for (let i = 0; i < gGridPrefs.gridColumns; i++) {
      row.appendChild(cell.cloneNode(true));
    }

    // Creates the grid
    for (let j = 0; j < gGridPrefs.gridRows; j++) {
      this._node.appendChild(row.cloneNode(true));
    }      

    // Create cell array.
    let cellElements = this.node.querySelectorAll(".newtab-cell");
    let cells = Array.from(cellElements, (cell) => new Cell(this, cell));

    // Fetch links.
    let links = gLinks.getLinks();

    // Create sites.
    let numLinks = Math.min(links.length, cells.length);
    for (let i = 0; i < numLinks; i++) {
      if (links[i]) {
        this.createSite(links[i], cells[i]);
      }
    }

    this._cells = cells;
  },

  /**
   * Creates the DOM fragment that is re-used when creating sites.
   */
  _createSiteFragment: function() {
    let site = document.createElementNS(HTML_NAMESPACE, "div");
    site.classList.add("newtab-site");
    site.setAttribute("draggable", "true");

    // Create the site's inner HTML code.
    site.innerHTML =
      '<a class="newtab-link">' +
      '  <span class="newtab-thumbnail placeholder"/>' +
      '  <span class="newtab-thumbnail thumbnail"/>' +
      '  <span class="newtab-title"/>' +
      '</a>' +
      '<input type="button" title="' + newTabString("pin") + '"' +
      '       class="newtab-control newtab-control-pin"/>' +
      '<input type="button" title="' + newTabString("block") + '"' +
      '       class="newtab-control newtab-control-block"/>';

    this._siteFragment = document.createDocumentFragment();
    this._siteFragment.appendChild(site);
  },

  /**
   * Test a tile at a given position for being pinned or history
   * @param position Position in sites array
   */
  _isHistoricalTile: function(aPos) {
    let site = this.sites[aPos];
    return site && (site.isPinned() || site.link && site.link.type == "history");
  }

};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\cells.js"

/**
 * This class manages a cell's DOM node (not the actually cell content, a site).
 * It's mostly read-only, i.e. all manipulation of both position and content
 * aren't handled here.
 */
function Cell(aGrid, aNode) {
  this._grid = aGrid;
  this._node = aNode;
  this._node._newtabCell = this;

  // Register drag-and-drop event handlers.
  ["dragenter", "dragover", "dragexit", "drop"].forEach(function(aType) {
    this._node.addEventListener(aType, this, false);
  }, this);
}

Cell.prototype = {
  /**
   * The grid.
   */
  _grid: null,

  /**
   * The cell's DOM node.
   */
  get node() { return this._node; },

  /**
   * The cell's offset in the grid.
   */
  get index() {
    let index = this._grid.cells.indexOf(this);

    // Cache this value, overwrite the getter.
    Object.defineProperty(this, "index", {value: index, enumerable: true});

    return index;
  },

  /**
   * The previous cell in the grid.
   */
  get previousSibling() {
    let prev = this.node.previousElementSibling;
    prev = prev && prev._newtabCell;

    // Cache this value, overwrite the getter.
    Object.defineProperty(this, "previousSibling", {value: prev, enumerable: true});

    return prev;
  },

  /**
   * The next cell in the grid.
   */
  get nextSibling() {
    let next = this.node.nextElementSibling;
    next = next && next._newtabCell;

    // Cache this value, overwrite the getter.
    Object.defineProperty(this, "nextSibling", {value: next, enumerable: true});

    return next;
  },

  /**
   * The site contained in the cell, if any.
   */
  get site() {
    let firstChild = this.node.firstElementChild;
    return firstChild && firstChild._newtabSite;
  },

  /**
   * Checks whether the cell contains a pinned site.
   * @return Whether the cell contains a pinned site.
   */
  containsPinnedSite: function() {
    let site = this.site;
    return site && site.isPinned();
  },

  /**
   * Checks whether the cell contains a site (is empty).
   * @return Whether the cell is empty.
   */
  isEmpty: function() {
    return !this.site;
  },

  /**
   * Handles all cell events.
   */
  handleEvent: function(aEvent) {
    // We're not responding to external drag/drop events
    // when our parent window is in private browsing mode.
    if (inPrivateBrowsingMode() && !gDrag.draggedSite)
      return;

    if (aEvent.type != "dragexit" && !gDrag.isValid(aEvent))
      return;

    switch (aEvent.type) {
      case "dragenter":
        aEvent.preventDefault();
        gDrop.enter(this, aEvent);
        break;
      case "dragover":
        aEvent.preventDefault();
        break;
      case "dragexit":
        gDrop.exit(this, aEvent);
        break;
      case "drop":
        aEvent.preventDefault();
        gDrop.drop(this, aEvent);
        break;
    }
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\sites.js"

const THUMBNAIL_PLACEHOLDER_ENABLED =
  Services.prefs.getBoolPref("browser.newtabpage.thumbnailPlaceholder");

/**
 * This class represents a site that is contained in a cell and can be pinned,
 * moved around or deleted.
 */
function Site(aNode, aLink) {
  this._node = aNode;
  this._node._newtabSite = this;

  this._link = aLink;

  this._render();
  this._addEventHandlers();
}

Site.prototype = {
  /**
   * The site's DOM node.
   */
  get node() { return this._node; },

  /**
   * The site's link.
   */
  get link() { return this._link; },

  /**
   * The url of the site's link.
   */
  get url() { return this.link.url; },

  /**
   * The title of the site's link.
   */
  get title() { return this.link.title || this.link.url; },

  /**
   * The site's parent cell.
   */
  get cell() {
    let parentNode = this.node.parentNode;
    return parentNode && parentNode._newtabCell;
  },

  /**
   * Pins the site on its current or a given index.
   * @param aIndex The pinned index (optional).
   * @return true if link changed type after pin
   */
  pin: function(aIndex) {
    if (typeof aIndex == "undefined")
      aIndex = this.cell.index;

    this._updateAttributes(true);
    let changed = gPinnedLinks.pin(this._link, aIndex);
    if (changed) {
      // render site again
      this._render();
    }
    return changed;
  },

  /**
   * Unpins the site and calls the given callback when done.
   */
  unpin: function() {
    if (this.isPinned()) {
      this._updateAttributes(false);
      gPinnedLinks.unpin(this._link);
      gUpdater.updateGrid();
    }
  },

  /**
   * Checks whether this site is pinned.
   * @return Whether this site is pinned.
   */
  isPinned: function() {
    return gPinnedLinks.isPinned(this._link);
  },

  /**
   * Blocks the site (removes it from the grid) and calls the given callback
   * when done.
   */
  block: function() {
    if (!gBlockedLinks.isBlocked(this._link)) {
      gUndoDialog.show(this);
      gBlockedLinks.block(this._link);
      gUpdater.updateGrid();
    }
  },

  /**
   * Gets the DOM node specified by the given query selector.
   * @param aSelector The query selector.
   * @return The DOM node we found.
   */
  _querySelector: function(aSelector) {
    return this.node.querySelector(aSelector);
  },

  /**
   * Updates attributes for all nodes which status depends on this site being
   * pinned or unpinned.
   * @param aPinned Whether this site is now pinned or unpinned.
   */
  _updateAttributes: function(aPinned) {
    let control = this._querySelector(".newtab-control-pin");

    if (aPinned) {
      this.node.setAttribute("pinned", true);
      control.setAttribute("title", newTabString("unpin"));
    } else {
      this.node.removeAttribute("pinned");
      control.setAttribute("title", newTabString("pin"));
    }
  },

  _newTabString: function(str, substrArr) {
    let regExp = /%[0-9]\$S/g;
    let matches;
    while ((matches = regExp.exec(str))) {
      let match = matches[0];
      let index = match.charAt(1); // Get the digit in the regExp.
      str = str.replace(match, substrArr[index - 1]);
    }
    return str;
  },

  /**
   * Checks for and modifies link at campaign end time
   */
  _checkLinkEndTime: function() {
    if (this.link.endTime && this.link.endTime < Date.now()) {
       let oldUrl = this.url;
       // chop off the path part from url
       this.link.url = Services.io.newURI(this.url, null, null).resolve("/");
       // clear supplied images - this triggers thumbnail download for new url
       delete this.link.imageURI;
       // remove endTime to avoid further time checks
       delete this.link.endTime;
       gPinnedLinks.replace(oldUrl, this.link);
    }
  },

  /**
   * Renders the site's data (fills the HTML fragment).
   */
  _render: function() {
    // first check for end time, as it may modify the link
    this._checkLinkEndTime();
    // setup display variables
    let url = this.url;
    let title = this.link.type == "history" ? this.link.baseDomain :
                this.title;
    let tooltip = (this.title == url ? this.title : this.title + "\n" + url);

    let link = this._querySelector(".newtab-link");
    link.setAttribute("title", tooltip);
    link.setAttribute("href", url);
    this.node.setAttribute("type", this.link.type);

    let titleNode = this._querySelector(".newtab-title");
    titleNode.textContent = title;
    if (this.link.titleBgColor) {
      titleNode.style.backgroundColor = this.link.titleBgColor;
    }

    if (this.isPinned())
      this._updateAttributes(true);
    // Capture the page if the thumbnail is missing, which will cause page.js
    // to be notified and call our refreshThumbnail() method.
    this.captureIfMissing();
    // but still display whatever thumbnail might be available now.
    this.refreshThumbnail();
  },

  /**
   * Called when the site's tab becomes visible for the first time.
   * Since the newtab may be preloaded long before it's displayed,
   * check for changed conditions and re-render if needed
   */
  onFirstVisible: function() {
    if (this.link.endTime && this.link.endTime < Date.now()) {
      // site needs to change landing url and background image
      this._render();
    }
    else {
      this.captureIfMissing();
    }
  },

  /**
   * Captures the site's thumbnail in the background, but only if there's no
   * existing thumbnail and the page allows background captures.
   */
  captureIfMissing: function() {
    if (!document.hidden && !this.link.imageURI) {
      BackgroundPageThumbs.captureIfMissing(this.url);
    }
  },

  /**
   * Refreshes the thumbnail for the site.
   */
  refreshThumbnail: function() {
    let link = this.link;

    let thumbnail = this._querySelector(".newtab-thumbnail.thumbnail");
    if (link.bgColor) {
      thumbnail.style.backgroundColor = link.bgColor;
    }
    let uri = link.imageURI || PageThumbs.getThumbnailURL(this.url);
    thumbnail.style.backgroundImage = 'url("' + uri + '")';

    if (THUMBNAIL_PLACEHOLDER_ENABLED &&
        link.type == "history" &&
        link.baseDomain) {
      let placeholder = this._querySelector(".newtab-thumbnail.placeholder");
      let charCodeSum = 0;
      for (let c of link.baseDomain) {
        charCodeSum += c.charCodeAt(0);
      }
      const COLORS = 16;
      let hue = Math.round((charCodeSum % COLORS) / COLORS * 360);
      placeholder.style.backgroundColor = "hsl(" + hue + ",80%,40%)";
      placeholder.textContent = link.baseDomain.substr(0,1).toUpperCase();
    }
  },

  _ignoreHoverEvents: function(element) {
    element.addEventListener("mouseover", () => {
      this.cell.node.setAttribute("ignorehover", "true");
    });
    element.addEventListener("mouseout", () => {
      this.cell.node.removeAttribute("ignorehover");
    });
  },

  /**
   * Adds event handlers for the site and its buttons.
   */
  _addEventHandlers: function() {
    // Register drag-and-drop event handlers.
    this._node.addEventListener("dragstart", this, false);
    this._node.addEventListener("dragend", this, false);
    this._node.addEventListener("mouseover", this, false);
  },

  /**
   * Speculatively opens a connection to the current site.
   */
  _speculativeConnect: function() {
    let sc = Services.io.QueryInterface(Ci.nsISpeculativeConnect);
    let uri = Services.io.newURI(this.url, null, null);
    try {
      // This can throw for certain internal URLs, when they wind up in
      // about:newtab. Be sure not to propagate the error.
      sc.speculativeConnect(uri, null);
    } catch (e) {}
  },

  _toggleLegalText: function(buttonClass, explanationTextClass) {
    let button = this._querySelector(buttonClass);
    if (button.hasAttribute("active")) {
      let explain = this._querySelector(explanationTextClass);
      explain.parentNode.removeChild(explain);

      button.removeAttribute("active");
    }
  },

  /**
   * Handles site click events.
   */
  onClick: function(aEvent) {
    let action;
    let pinned = this.isPinned();
    let tileIndex = this.cell.index;
    let {button, target} = aEvent;

    // Handle tile/thumbnail link click
    if (target.classList.contains("newtab-link") ||
        target.parentElement.classList.contains("newtab-link")) {
      // Record for primary and middle clicks
      if (button == 0 || button == 1) {
        action = "click";
      }
    }
    // Only handle primary clicks for the remaining targets
    else if (button == 0) {
      aEvent.preventDefault();
      if (target.classList.contains("newtab-control-block")) {
        this.block();
        action = "block";
      }
      else if (pinned && target.classList.contains("newtab-control-pin")) {
        this.unpin();
        action = "unpin";
      }
      else if (!pinned && target.classList.contains("newtab-control-pin")) {
        if (this.pin()) {
          // link has changed - update rest of the pages
          gAllPages.update(gPage);
        }
        action = "pin";
      }
    }
  },

  /**
   * Handles all site events.
   */
  handleEvent: function(aEvent) {
    switch (aEvent.type) {
      case "mouseover":
        this._node.removeEventListener("mouseover", this, false);
        this._speculativeConnect();
        break;
      case "dragstart":
        gDrag.start(this, aEvent);
        break;
      case "dragend":
        gDrag.end(this, aEvent);
        break;
    }
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\drag.js"

/**
 * This singleton implements site dragging functionality.
 */
var gDrag = {
  /**
   * The site offset to the drag start point.
   */
  _offsetX: null,
  _offsetY: null,

  /**
   * The site that is dragged.
   */
  _draggedSite: null,
  get draggedSite() { return this._draggedSite; },

  /**
   * The cell width/height at the point the drag started.
   */
  _cellWidth: null,
  _cellHeight: null,
  get cellWidth() { return this._cellWidth; },
  get cellHeight() { return this._cellHeight; },

  /**
   * Start a new drag operation.
   * @param aSite The site that's being dragged.
   * @param aEvent The 'dragstart' event.
   */
  start: function(aSite, aEvent) {
    this._draggedSite = aSite;

    // Mark nodes as being dragged.
    let selector = ".newtab-site, .newtab-control, .newtab-thumbnail";
    let parentCell = aSite.node.parentNode;
    let nodes = parentCell.querySelectorAll(selector);
    for (let i = 0; i < nodes.length; i++)
      nodes[i].setAttribute("dragged", "true");

    parentCell.setAttribute("dragged", "true");

    this._setDragData(aSite, aEvent);

    // Store the cursor offset.
    let node = aSite.node;
    let rect = node.getBoundingClientRect();
    this._offsetX = aEvent.clientX - rect.left;
    this._offsetY = aEvent.clientY - rect.top;

    // Store the cell dimensions.
    let cellNode = aSite.cell.node;
    this._cellWidth = cellNode.offsetWidth;
    this._cellHeight = cellNode.offsetHeight;

    gTransformation.freezeSitePosition(aSite);
  },

  /**
   * Handles the 'drag' event.
   * @param aSite The site that's being dragged.
   * @param aEvent The 'drag' event.
   */
  drag: function(aSite, aEvent) {
    // Get the viewport size.
    let {clientWidth, clientHeight} = document.documentElement;

    // We'll want a padding of 5px.
    let border = 5;

    // Enforce minimum constraints to keep the drag image inside the window.
    let left = Math.max(scrollX + aEvent.clientX - this._offsetX, border);
    let top = Math.max(scrollY + aEvent.clientY - this._offsetY, border);

    // Enforce maximum constraints to keep the drag image inside the window.
    left = Math.min(left, scrollX + clientWidth - this.cellWidth - border);
    top = Math.min(top, scrollY + clientHeight - this.cellHeight - border);

    // Update the drag image's position.
    gTransformation.setSitePosition(aSite, {left: left, top: top});
  },

  /**
   * Ends the current drag operation.
   * @param aSite The site that's being dragged.
   * @param aEvent The 'dragend' event.
   */
  end: function(aSite, aEvent) {
    let nodes = gGrid.node.querySelectorAll("[dragged]")
    for (let i = 0; i < nodes.length; i++)
      nodes[i].removeAttribute("dragged");

    // Slide the dragged site back into its cell (may be the old or the new cell).
    gTransformation.slideSiteTo(aSite, aSite.cell, {unfreeze: true});

    this._draggedSite = null;
  },

  /**
   * Checks whether we're responsible for a given drag event.
   * @param aEvent The drag event to check.
   * @return Whether we should handle this drag and drop operation.
   */
  isValid: function(aEvent) {
    let link = gDragDataHelper.getLinkFromDragEvent(aEvent);

    // Check that the drag data is non-empty.
    // Can happen when dragging places folders.
    if (!link || !link.url) {
      return false;
    }

    // Check that we're not accepting URLs which would inherit the caller's
    // principal (such as javascript: or data:).
    return gLinkChecker.checkLoadURI(link.url);
  },

  /**
   * Initializes the drag data for the current drag operation.
   * @param aSite The site that's being dragged.
   * @param aEvent The 'dragstart' event.
   */
  _setDragData: function(aSite, aEvent) {
    let {url, title} = aSite;

    let dt = aEvent.dataTransfer;
    dt.mozCursor = "default";
    dt.effectAllowed = "move";
    dt.setData("text/plain", url);
    dt.setData("text/uri-list", url);
    dt.setData("text/x-moz-url", url + "\n" + title);
    dt.setData("text/html", "<a href=\"" + url + "\">" + url + "</a>");

    // Create and use an empty drag element. We don't want to use the default
    // drag image with its default opacity.
    let dragElement = document.createElementNS(HTML_NAMESPACE, "div");
    dragElement.classList.add("newtab-drag");
    let scrollbox = document.getElementById("newtab-vertical-margin");
    scrollbox.appendChild(dragElement);
    dt.setDragImage(dragElement, 0, 0);

    // After the 'dragstart' event has been processed we can remove the
    // temporary drag element from the DOM.
    setTimeout(() => scrollbox.removeChild(dragElement), 0);
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\dragDataHelper.js"

var gDragDataHelper = {
  get mimeType() {
    return "text/x-moz-url";
  },

  getLinkFromDragEvent: function(aEvent) {
    let dt = aEvent.dataTransfer;
    if (!dt || !dt.types.includes(this.mimeType)) {
      return null;
    }

    let data = dt.getData(this.mimeType) || "";
    let [url, title] = data.split(/[\r\n]+/);
    return {url: url, title: title};
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\drop.js"

// A little delay that prevents the grid from being too sensitive when dragging
// sites around.
const DELAY_REARRANGE_MS = 100;

/**
 * This singleton implements site dropping functionality.
 */
var gDrop = {
  /**
   * The last drop target.
   */
  _lastDropTarget: null,

  /**
   * Handles the 'dragenter' event.
   * @param aCell The drop target cell.
   */
  enter: function(aCell) {
    this._delayedRearrange(aCell);
  },

  /**
   * Handles the 'dragexit' event.
   * @param aCell The drop target cell.
   * @param aEvent The 'dragexit' event.
   */
  exit: function(aCell, aEvent) {
    if (aEvent.dataTransfer && !aEvent.dataTransfer.mozUserCancelled) {
      this._delayedRearrange();
    } else {
      // The drag operation has been cancelled.
      this._cancelDelayedArrange();
      this._rearrange();
    }
  },

  /**
   * Handles the 'drop' event.
   * @param aCell The drop target cell.
   * @param aEvent The 'dragexit' event.
   */
  drop: function(aCell, aEvent) {
    // The cell that is the drop target could contain a pinned site. We need
    // to find out where that site has gone and re-pin it there.
    if (aCell.containsPinnedSite())
      this._repinSitesAfterDrop(aCell);

    // Pin the dragged or insert the new site.
    this._pinDraggedSite(aCell, aEvent);

    this._cancelDelayedArrange();

    // Update the grid and move all sites to their new places.
    gUpdater.updateGrid();
  },

  /**
   * Re-pins all pinned sites in their (new) positions.
   * @param aCell The drop target cell.
   */
  _repinSitesAfterDrop: function(aCell) {
    let sites = gDropPreview.rearrange(aCell);

    // Filter out pinned sites.
    let pinnedSites = sites.filter(function(aSite) {
      return aSite && aSite.isPinned();
    });

    // Re-pin all shifted pinned cells.
    pinnedSites.forEach(aSite => aSite.pin(sites.indexOf(aSite)));
  },

  /**
   * Pins the dragged site in its new place.
   * @param aCell The drop target cell.
   * @param aEvent The 'dragexit' event.
   */
  _pinDraggedSite: function(aCell, aEvent) {
    let index = aCell.index;
    let draggedSite = gDrag.draggedSite;

    if (draggedSite) {
      // Pin the dragged site at its new place.
      if (aCell != draggedSite.cell)
        draggedSite.pin(index);
    } else {
      let link = gDragDataHelper.getLinkFromDragEvent(aEvent);
      if (link) {
        // A new link was dragged onto the grid. Create it by pinning its URL.
        gPinnedLinks.pin(link, index);

        // Make sure the newly added link is not blocked.
        gBlockedLinks.unblock(link);
      }
    }
  },

  /**
   * Time a rearrange with a little delay.
   * @param aCell The drop target cell.
   */
  _delayedRearrange: function(aCell) {
    // The last drop target didn't change so there's no need to re-arrange.
    if (this._lastDropTarget == aCell)
      return;

    let self = this;

    function callback() {
      self._rearrangeTimeout = null;
      self._rearrange(aCell);
    }

    this._cancelDelayedArrange();
    this._rearrangeTimeout = setTimeout(callback, DELAY_REARRANGE_MS);

    // Store the last drop target.
    this._lastDropTarget = aCell;
  },

  /**
   * Cancels a timed rearrange, if any.
   */
  _cancelDelayedArrange: function() {
    if (this._rearrangeTimeout) {
      clearTimeout(this._rearrangeTimeout);
      this._rearrangeTimeout = null;
    }
  },

  /**
   * Rearrange all sites in the grid depending on the current drop target.
   * @param aCell The drop target cell.
   */
  _rearrange: function(aCell) {
    let sites = gGrid.sites;

    // We need to rearrange the grid only if there's a current drop target.
    if (aCell)
      sites = gDropPreview.rearrange(aCell);

    gTransformation.rearrangeSites(sites, {unfreeze: !aCell});
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\dropTargetShim.js"

/**
 * This singleton provides a custom drop target detection. We need this because
 * the default DnD target detection relies on the cursor's position. We want
 * to pick a drop target based on the dragged site's position.
 */
var gDropTargetShim = {
  /**
   * Cache for the position of all cells, cleaned after drag finished.
   */
  _cellPositions: null,

  /**
   * The last drop target that was hovered.
   */
  _lastDropTarget: null,

  /**
   * Initializes the drop target shim.
   */
  init: function() {
    gGrid.node.addEventListener("dragstart", this, true);
  },

  /**
   * Add all event listeners needed during a drag operation.
   */
  _addEventListeners: function() {
    gGrid.node.addEventListener("dragend", this);

    let docElement = document.documentElement;
    docElement.addEventListener("dragover", this);
    docElement.addEventListener("dragenter", this);
    docElement.addEventListener("drop", this);
  },

  /**
   * Remove all event listeners that were needed during a drag operation.
   */
  _removeEventListeners: function() {
    gGrid.node.removeEventListener("dragend", this);

    let docElement = document.documentElement;
    docElement.removeEventListener("dragover", this);
    docElement.removeEventListener("dragenter", this);
    docElement.removeEventListener("drop", this);
  },

  /**
   * Handles all shim events.
   */
  handleEvent: function(aEvent) {
    switch (aEvent.type) {
      case "dragstart":
        this._dragstart(aEvent);
        break;
      case "dragenter":
        aEvent.preventDefault();
        break;
      case "dragover":
        this._dragover(aEvent);
        break;
      case "drop":
        this._drop(aEvent);
        break;
      case "dragend":
        this._dragend(aEvent);
        break;
    }
  },

  /**
   * Handles the 'dragstart' event.
   * @param aEvent The 'dragstart' event.
   */
  _dragstart: function(aEvent) {
    if (aEvent.target.classList.contains("newtab-link")) {
      gGrid.lock();
      this._addEventListeners();
    }
  },

  /**
   * Handles the 'dragover' event.
   * @param aEvent The 'dragover' event.
   */
  _dragover: function(aEvent) {
    // XXX bug 505521 - Use the dragover event to retrieve the
    //                  current mouse coordinates while dragging.
    let sourceNode = aEvent.dataTransfer.mozSourceNode.parentNode;
    gDrag.drag(sourceNode._newtabSite, aEvent);

    // Find the current drop target, if there's one.
    this._updateDropTarget(aEvent);

    // If we have a valid drop target,
    // let the drag-and-drop service know.
    if (this._lastDropTarget) {
      aEvent.preventDefault();
    }
  },

  /**
   * Handles the 'drop' event.
   * @param aEvent The 'drop' event.
   */
  _drop: function(aEvent) {
    // We're accepting all drops.
    aEvent.preventDefault();

    // remember that drop event was seen, this explicitly
    // assumes that drop event preceeds dragend event
    this._dropSeen = true;

    // Make sure to determine the current drop target
    // in case the dragover event hasn't been fired.
    this._updateDropTarget(aEvent);

    // A site was successfully dropped.
    this._dispatchEvent(aEvent, "drop", this._lastDropTarget);
  },

  /**
   * Handles the 'dragend' event.
   * @param aEvent The 'dragend' event.
   */
  _dragend: function(aEvent) {
    if (this._lastDropTarget) {
      if (aEvent.dataTransfer.mozUserCancelled || !this._dropSeen) {
        // The drag operation was cancelled or no drop event was generated
        this._dispatchEvent(aEvent, "dragexit", this._lastDropTarget);
        this._dispatchEvent(aEvent, "dragleave", this._lastDropTarget);
      }

      // Clean up.
      this._lastDropTarget = null;
      this._cellPositions = null;
    }

    this._dropSeen = false;
    gGrid.unlock();
    this._removeEventListeners();
  },

  /**
   * Tries to find the current drop target and will fire
   * appropriate dragenter, dragexit, and dragleave events.
   * @param aEvent The current drag event.
   */
  _updateDropTarget: function(aEvent) {
    // Let's see if we find a drop target.
    let target = this._findDropTarget(aEvent);

    if (target != this._lastDropTarget) {
      if (this._lastDropTarget)
        // We left the last drop target.
        this._dispatchEvent(aEvent, "dragexit", this._lastDropTarget);

      if (target)
        // We're now hovering a (new) drop target.
        this._dispatchEvent(aEvent, "dragenter", target);

      if (this._lastDropTarget)
        // We left the last drop target.
        this._dispatchEvent(aEvent, "dragleave", this._lastDropTarget);

      this._lastDropTarget = target;
    }
  },

  /**
   * Determines the current drop target by matching the dragged site's position
   * against all cells in the grid.
   * @return The currently hovered drop target or null.
   */
  _findDropTarget: function() {
    // These are the minimum intersection values - we want to use the cell if
    // the site is >= 50% hovering its position.
    let minWidth = gDrag.cellWidth / 2;
    let minHeight = gDrag.cellHeight / 2;

    let cellPositions = this._getCellPositions();
    let rect = gTransformation.getNodePosition(gDrag.draggedSite.node);

    // Compare each cell's position to the dragged site's position.
    for (let i = 0; i < cellPositions.length; i++) {
      let inter = rect.intersect(cellPositions[i].rect);

      // If the intersection is big enough we found a drop target.
      if (inter.width >= minWidth && inter.height >= minHeight)
        return cellPositions[i].cell;
    }

    // No drop target found.
    return null;
  },

  /**
   * Gets the positions of all cell nodes.
   * @return The (cached) cell positions.
   */
  _getCellPositions: function() {
    if (this._cellPositions)
      return this._cellPositions;

    return this._cellPositions = gGrid.cells.map(function(cell) {
      return {cell: cell, rect: gTransformation.getNodePosition(cell.node)};
    });
  },

  /**
   * Dispatches a custom DragEvent on the given target node.
   * @param aEvent The source event.
   * @param aType The event type.
   * @param aTarget The target node that receives the event.
   */
  _dispatchEvent: function(aEvent, aType, aTarget) {
    let node = aTarget.node;
    let event = document.createEvent("DragEvent");

    // The event should not bubble to prevent recursion.
    event.initDragEvent(aType, false, true, window, 0, 0, 0, 0, 0, false, false,
                        false, false, 0, node, aEvent.dataTransfer);

    node.dispatchEvent(event);
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\dropPreview.js"

/**
 * This singleton provides the ability to re-arrange the current grid to
 * indicate the transformation that results from dropping a cell at a certain
 * position.
 */
var gDropPreview = {
  /**
   * Rearranges the sites currently contained in the grid when a site would be
   * dropped onto the given cell.
   * @param aCell The drop target cell.
   * @return The re-arranged array of sites.
   */
  rearrange: function(aCell) {
    let sites = gGrid.sites;

    // Insert the dragged site into the current grid.
    this._insertDraggedSite(sites, aCell);

    // After the new site has been inserted we need to correct the positions
    // of all pinned tabs that have been moved around.
    this._repositionPinnedSites(sites, aCell);

    return sites;
  },

  /**
   * Inserts the currently dragged site into the given array of sites.
   * @param aSites The array of sites to insert into.
   * @param aCell The drop target cell.
   */
  _insertDraggedSite: function(aSites, aCell) {
    let dropIndex = aCell.index;
    let draggedSite = gDrag.draggedSite;

    // We're currently dragging a site.
    if (draggedSite) {
      let dragCell = draggedSite.cell;
      let dragIndex = dragCell.index;

      // Move the dragged site into its new position.
      if (dragIndex != dropIndex) {
        aSites.splice(dragIndex, 1);
        aSites.splice(dropIndex, 0, draggedSite);
      }
    // We're handling an external drag item.
    } else {
      aSites.splice(dropIndex, 0, null);
    }
  },

  /**
   * Correct the position of all pinned sites that might have been moved to
   * different positions after the dragged site has been inserted.
   * @param aSites The array of sites containing the dragged site.
   * @param aCell The drop target cell.
   */
  _repositionPinnedSites:
    function(aSites, aCell) {

    // Collect all pinned sites.
    let pinnedSites = this._filterPinnedSites(aSites, aCell);

    // Correct pinned site positions.
    pinnedSites.forEach(function(aSite) {
      aSites[aSites.indexOf(aSite)] = aSites[aSite.cell.index];
      aSites[aSite.cell.index] = aSite;
    }, this);

    // There might be a pinned cell that got pushed out of the grid, try to
    // sneak it in by removing a lower-priority cell.
    if (this._hasOverflowedPinnedSite(aSites, aCell))
      this._repositionOverflowedPinnedSite(aSites, aCell);
  },

  /**
   * Filter pinned sites out of the grid that are still on their old positions
   * and have not moved.
   * @param aSites The array of sites to filter.
   * @param aCell The drop target cell.
   * @return The filtered array of sites.
   */
  _filterPinnedSites: function(aSites, aCell) {
    let draggedSite = gDrag.draggedSite;

    // When dropping on a cell that contains a pinned site make sure that all
    // pinned cells surrounding the drop target are moved as well.
    let range = this._getPinnedRange(aCell);

    return aSites.filter(function(aSite, aIndex) {
      // The site must be valid, pinned and not the dragged site.
      if (!aSite || aSite == draggedSite || !aSite.isPinned())
        return false;

      let index = aSite.cell.index;

      // If it's not in the 'pinned range' it's a valid pinned site.
      return (index > range.end || index < range.start);
    });
  },

  /**
   * Determines the range of pinned sites surrounding the drop target cell.
   * @param aCell The drop target cell.
   * @return The range of pinned cells.
   */
  _getPinnedRange: function(aCell) {
    let dropIndex = aCell.index;
    let range = {start: dropIndex, end: dropIndex};

    // We need a pinned range only when dropping on a pinned site.
    if (aCell.containsPinnedSite()) {
      let links = gPinnedLinks.links;

      // Find all previous siblings of the drop target that are pinned as well.
      while (range.start && links[range.start - 1])
        range.start--;

      let maxEnd = links.length - 1;

      // Find all next siblings of the drop target that are pinned as well.
      while (range.end < maxEnd && links[range.end + 1])
        range.end++;
    }

    return range;
  },

  /**
   * Checks if the given array of sites contains a pinned site that has
   * been pushed out of the grid.
   * @param aSites The array of sites to check.
   * @param aCell The drop target cell.
   * @return Whether there is an overflowed pinned cell.
   */
  _hasOverflowedPinnedSite:
    function(aSites, aCell) {

    // If the drop target isn't pinned there's no way a pinned site has been
    // pushed out of the grid so we can just exit here.
    if (!aCell.containsPinnedSite())
      return false;

    let cells = gGrid.cells;

    // No cells have been pushed out of the grid, nothing to do here.
    if (aSites.length <= cells.length)
      return false;

    let overflowedSite = aSites[cells.length];

    // Nothing to do if the site that got pushed out of the grid is not pinned.
    return (overflowedSite && overflowedSite.isPinned());
  },

  /**
   * We have a overflowed pinned site that we need to re-position so that it's
   * visible again. We try to find a lower-priority cell (empty or containing
   * an unpinned site) that we can move it to.
   * @param aSites The array of sites.
   * @param aCell The drop target cell.
   */
  _repositionOverflowedPinnedSite:
    function(aSites, aCell) {

    // Try to find a lower-priority cell (empty or containing an unpinned site).
    let index = this._indexOfLowerPrioritySite(aSites, aCell);

    if (index > -1) {
      let cells = gGrid.cells;
      let dropIndex = aCell.index;

      // Move all pinned cells to their new positions to let the overflowed
      // site fit into the grid.
      for (let i = index + 1, lastPosition = index; i < aSites.length; i++) {
        if (i != dropIndex) {
          aSites[lastPosition] = aSites[i];
          lastPosition = i;
        }
      }

      // Finally, remove the overflowed site from its previous position.
      aSites.splice(cells.length, 1);
    }
  },

  /**
   * Finds the index of the last cell that is empty or contains an unpinned
   * site. These are considered to be of a lower priority.
   * @param aSites The array of sites.
   * @param aCell The drop target cell.
   * @return The cell's index.
   */
  _indexOfLowerPrioritySite:
    function(aSites, aCell) {

    let cells = gGrid.cells;
    let dropIndex = aCell.index;

    // Search (beginning with the last site in the grid) for a site that is
    // empty or unpinned (an thus lower-priority) and can be pushed out of the
    // grid instead of the pinned site.
    for (let i = cells.length - 1; i >= 0; i--) {
      // The cell that is our drop target is not a good choice.
      if (i == dropIndex)
        continue;

      let site = aSites[i];

      // We can use the cell only if it's empty or the site is un-pinned.
      if (!site || !site.isPinned())
        return i;
    }

    return -1;
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\updater.js"

/**
 * This singleton provides functionality to update the current grid to a new
 * set of pinned and blocked sites. It adds, moves and removes sites.
 */
var gUpdater = {
  /**
   * Updates the current grid according to its pinned and blocked sites.
   * This removes old, moves existing and creates new sites to fill gaps.
   * @param aCallback The callback to call when finished.
   */
  updateGrid: function(aCallback) {
    let links = gLinks.getLinks().slice(0, gGrid.cells.length);

    // Find all sites that remain in the grid.
    let sites = this._findRemainingSites(links);

    // Remove sites that are no longer in the grid.
    this._removeLegacySites(sites, () => {
      // Freeze all site positions so that we can move their DOM nodes around
      // without any visual impact.
      this._freezeSitePositions(sites);

      // Move the sites' DOM nodes to their new position in the DOM. This will
      // have no visual effect as all the sites have been frozen and will
      // remain in their current position.
      this._moveSiteNodes(sites);

      // Now it's time to animate the sites actually moving to their new
      // positions.
      this._rearrangeSites(sites, () => {
        // Try to fill empty cells and finish.
        this._fillEmptyCells(links, aCallback);

        // Update other pages that might be open to keep them synced.
        gAllPages.update(gPage);
      });
    });
  },

  /**
   * Takes an array of links and tries to correlate them to sites contained in
   * the current grid. If no corresponding site can be found (i.e. the link is
   * new and a site will be created) then just set it to null.
   * @param aLinks The array of links to find sites for.
   * @return Array of sites mapped to the given links (can contain null values).
   */
  _findRemainingSites: function(aLinks) {
    let map = {};

    // Create a map to easily retrieve the site for a given URL.
    gGrid.sites.forEach(function(aSite) {
      if (aSite)
        map[aSite.url] = aSite;
    });

    // Map each link to its corresponding site, if any.
    return aLinks.map(function(aLink) {
      return aLink && (aLink.url in map) && map[aLink.url];
    });
  },

  /**
   * Freezes the given sites' positions.
   * @param aSites The array of sites to freeze.
   */
  _freezeSitePositions: function(aSites) {
    aSites.forEach(function(aSite) {
      if (aSite)
        gTransformation.freezeSitePosition(aSite);
    });
  },

  /**
   * Moves the given sites' DOM nodes to their new positions.
   * @param aSites The array of sites to move.
   */
  _moveSiteNodes: function(aSites) {
    let cells = gGrid.cells;

    // Truncate the given array of sites to not have more sites than cells.
    // This can happen when the user drags a bookmark (or any other new kind
    // of link) onto the grid.
    let sites = aSites.slice(0, cells.length);

    sites.forEach(function(aSite, aIndex) {
      let cell = cells[aIndex];
      let cellSite = cell.site;

      // The site's position didn't change.
      if (!aSite || cellSite != aSite) {
        let cellNode = cell.node;

        // Empty the cell if necessary.
        if (cellSite)
          cellNode.removeChild(cellSite.node);

        // Put the new site in place, if any.
        if (aSite)
          cellNode.appendChild(aSite.node);
      }
    }, this);
  },

  /**
   * Rearranges the given sites and slides them to their new positions.
   * @param aSites The array of sites to re-arrange.
   * @param aCallback The callback to call when finished.
   */
  _rearrangeSites: function(aSites, aCallback) {
    let options = {callback: aCallback, unfreeze: true};
    gTransformation.rearrangeSites(aSites, options);
  },

  /**
   * Removes all sites from the grid that are not in the given links array or
   * exceed the grid.
   * @param aSites The array of sites remaining in the grid.
   * @param aCallback The callback to call when finished.
   */
  _removeLegacySites: function(aSites, aCallback) {
    let batch = [];

    // Delete sites that were removed from the grid.
    gGrid.sites.forEach(function(aSite) {
      // The site must be valid and not in the current grid.
      if (!aSite || aSites.indexOf(aSite) != -1)
        return;

      batch.push(new Promise(resolve => {
        // Fade out the to-be-removed site.
        gTransformation.hideSite(aSite, function() {
          let node = aSite.node;

          // Remove the site from the DOM.
          node.parentNode.removeChild(node);
          resolve();
        });
      }));
    });

    Promise.all(batch).then(aCallback);
  },

  /**
   * Tries to fill empty cells with new links if available.
   * @param aLinks The array of links.
   * @param aCallback The callback to call when finished.
   */
  _fillEmptyCells: function(aLinks, aCallback) {
    let {cells, sites} = gGrid;

    // Find empty cells and fill them.
    Promise.all(sites.map((aSite, aIndex) => {
      if (aSite || !aLinks[aIndex])
        return null;

      return new Promise(resolve => {
        // Create the new site and fade it in.
        let site = gGrid.createSite(aLinks[aIndex], cells[aIndex]);

        // Set the site's initial opacity to zero.
        site.node.style.opacity = 0;

        // Flush all style changes for the dynamically inserted site to make
        // the fade-in transition work.
        window.getComputedStyle(site.node).opacity;
        gTransformation.showSite(site, resolve);
      });
    })).then(aCallback).catch(console.exception);
  }
};
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\undo.js"

/**
 * Dialog allowing to undo the removal of single site or to completely restore
 * the grid's original state.
 */
var gUndoDialog = {
  /**
   * The undo dialog's timeout in miliseconds.
   */
  HIDE_TIMEOUT_MS: 15000,

  /**
   * Contains undo information.
   */
  _undoData: null,

  /**
   * Initializes the undo dialog.
   */
  init: function() {
    this._undoContainer = document.getElementById("newtab-undo-container");
    this._undoContainer.addEventListener("click", this, false);
    this._undoButton = document.getElementById("newtab-undo-button");
    this._undoCloseButton = document.getElementById("newtab-undo-close-button");
    this._undoRestoreButton = document.getElementById("newtab-undo-restore-button");
  },

  /**
   * Shows the undo dialog.
   * @param aSite The site that just got removed.
   */
  show: function(aSite) {
    if (this._undoData)
      clearTimeout(this._undoData.timeout);

    this._undoData = {
      index: aSite.cell.index,
      wasPinned: aSite.isPinned(),
      blockedLink: aSite.link,
      timeout: setTimeout(this.hide.bind(this), this.HIDE_TIMEOUT_MS)
    };

    this._undoContainer.removeAttribute("undo-disabled");
    this._undoButton.removeAttribute("tabindex");
    this._undoCloseButton.removeAttribute("tabindex");
    this._undoRestoreButton.removeAttribute("tabindex");
  },

  /**
   * Hides the undo dialog.
   */
  hide: function() {
    if (!this._undoData)
      return;

    clearTimeout(this._undoData.timeout);
    this._undoData = null;
    this._undoContainer.setAttribute("undo-disabled", "true");
    this._undoButton.setAttribute("tabindex", "-1");
    this._undoCloseButton.setAttribute("tabindex", "-1");
    this._undoRestoreButton.setAttribute("tabindex", "-1");
  },

  /**
   * The undo dialog event handler.
   * @param aEvent The event to handle.
   */
  handleEvent: function(aEvent) {
    switch (aEvent.target.id) {
      case "newtab-undo-button":
        this._undo();
        break;
      case "newtab-undo-restore-button":
        this._undoAll();
        break;
      case "newtab-undo-close-button":
        this.hide();
        break;
    }
  },

  /**
   * Undo the last blocked site.
   */
  _undo: function() {
    if (!this._undoData)
      return;

    let {index, wasPinned, blockedLink} = this._undoData;
    gBlockedLinks.unblock(blockedLink);

    if (wasPinned) {
      gPinnedLinks.pin(blockedLink, index);
    }

    gUpdater.updateGrid();
    this.hide();
  },

  /**
   * Undo all blocked sites.
   */
  _undoAll: function() {
    NewTabUtils.undoAll(function() {
      gUpdater.updateGrid();
      this.hide();
    }.bind(this));
  }
};

gUndoDialog.init();
//@line 6 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\search.js"

//@line 1 "c:\binoczilla\src\binoc-central\ux\browser\components\shared\searchenginelogos.js"
/* This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at http://mozilla.org/MPL/2.0/. */

const SEARCH_ENGINES = {
  "DuckDuckGo": {
  	 image: "data:image/png;base64," +
            "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAMAAACdt4HsAAACT1BMVEXvISn/////9/fvUlr3ra3/" +
            "zs7/7+/va2v/5+f/xsbvMTn/tbX/3t7/vb3vOUL3WmPvQkr/zgDvKTHvSlL3hIT3paX/1tbnISn3" +
            "c3v3e3v3a3P3jIz3nJz/tb33c3PvKSn3lJT39/cAc73vSkr3e4Tv7+/3Yxj3pa3/tQj3jJT3nKX3" +
            "Y2P/xs73hIzvQkL/vQjvQiHn5+f3hBD/ztbvMTH/vcb/3ucIc733lJz/pQilzufe7/fvMSHOzs73" +
            "//cQrUpKvVprxmP3Y2vvShiUzmvWlJRzzmMYtUrvOTnn7/davVrWra3v9//nY2PvISGUxudztd7e" +
            "3t7/76XvKSHea2v/xgDnOUK93vfW5/f/1t73Uhj/52ut3q2l3rXO784pjMZrrdb/rQjera3/5+/e" +
            "paWMxufO79aEazkYrUr/nAj3jBD3axj3lBD///fehIRKpd7/1hCEYzk5vVL3//8ptVLW77UxtVLn" +
            "SlLW1tZCvVp7vef/1gj/3invSkL//+fWtbXvpaX/3kr/97XvnJznWmMxjM5zvefOxsbWnKXWjIzG" +
            "3u/ea3Pn997O5/fnQkqExuf3Whit1u/nUlrnxs7v5+d7zmuU1pT3exDOSjFjrVL/987/pUoQe8b/" +
            "75T/3jFKxnO158bWKSl7zoRSxmtajEK1e0pzxlqcUjH/1iHOMSnOvb33cxDWnJx7td6EzmP/74xz" +
            "azlrcznec3Pe771jxlpzczne78YpvVqEvWPn99YxvWOtSjHee3vG787OOTE5lEK1QjHv9+drzmve" +
            "tbXO772q+r8wAAAFbUlEQVR4Xo2X84PzTBDHN3Zqu2fbemzbNl7atm3btvGHvTNJ2myuyd3NL2mT" +
            "zmdnvjM76RImyGQlH5dCHBeSmscNmQkyfwBrZMLEY2aRF5cMSDYPEx+LZpUlAYRQbVEpnuc1je/M" +
            "SbVwYoVFAbpE0IaLmiwqiVymmE3H84YuGs2mheCEhQH5qPUrje2ONxHKVIkXR2x2MxsMkDnLvftk" +
            "2fSTQNCzSAgngwCCipkXxHiU+BsnCDFE8f6AQgnwaTGhkmDLymW8jPsBeIsth8iCpha618El1wgo" +
            "4FOhWyWLWY+O8pbnAwTI29S1ElncJBmF4L0AGeJSdR4dUpt5w+DL0nAgoUuGGKKCBxDCOxrykaDb" +
            "+yFQjhUylLlXpAB5jGnIqV6uvvWUcAAhLmDBXIAMrkXRdHQ+cerUiWefq1hRrAgg8LikUgdkQUAx" +
            "6+2Ze0WLEO/1BQzrHCFNrAPAeDSD4q/Ln6R3p68MSYzDAUiwIEutJM0bHXE/gpEhJMxaAB3T6aT8" +
            "mfkm+QBiMlwKFqAHvrHu9tvTOLrEdX4hFAkJWQB42qbVyam75ruv3zvF+wBCKJ0MAAV6SAy5+raA" +
            "y+lb9tYBUw9sffKRJh+CDl2SAEAPquaC76swU1c+zlxbA9if/EIY78AcCBODDKjnVzDM0+sb57zq" +
            "N14gdpbg4nraBaxm3NWpIDKNgJIIDTxEAKMyVM9/VrFcpijK52PbNhmk0RQORCA8dhGhIkDA+qPV" +
            "Y/U8No2NHZsUfQCdzYTECSiRSRJKgxYAnK6+tnVrPYL7q2P7GNNnT0L3SQSS61AowK4BAExWq9XJ" +
            "OmDT5D4GtUab7p92W1aD6AFBOjUKcONNKMG2o9vmScmhd+v5SCTS91StDLBwmHR5q0iiM4yv3X5g" +
            "sD1i24tUHc0GQOrOihdw+ZV7drx+8I1IzfpaCQ1oSIGsbqEBdxy8KkLb8dYt7m7AFBpEJI8OUIAd" +
            "Hve+wX509IqYgzLqxKMi5X+r6737wgHfMrZBKGwpQMWP0PN8/8qLn15cSRosEQeI3coxGrzRVfE2" +
            "BEyTAMNpmbA3k2erPOyq+CUCPGvv3OmGykYBQhiYFbynDLu2uyW826qb7bSlv/VCe2R3vQqhIYQQ" +
            "nLmSGKUAT1AqXn7V6p72iUsTThsNuhKUAeKMNFaiW2nG08H90IF1m6DywVdsHgA4bPgRGgAqUgBr" +
            "DwxOtPcdv9RK6yklnaGKOXBMmN7RVCtJJMiUdG2s78dv9HbY7KrI9AQBOHwjaxaA6cKhRLXCHkpF" +
            "PrAJYBz1su7LtSBQIjzozgI5AJDWsQ7gTJxETTHuEh5yW8kR5+1fvQBT5PDdWgPokE6GSuK3Aaby" +
            "2KwNyGFIZ8/NfexVMAGXEfe8MA5QTVdrgGe2M9evev6FMwiAYr308nVzcx/SgHwSlswyLgDLHU0K" +
            "tX5UZwCwZsM1b7516J1333v/g2UAuJoCNMsmZkEDZBXujCoOIfVJxQKsvXnDshvWfrEcAV9RAoqY" +
            "rfdvHjY06R3tVmtjzQYsQ8ByC/C1O0dEzqkAGqELbiZ1W/RvBr51Ad9ZgO8dQCkh4/q5xvMC6hot" +
            "sBl7rP1QT+HHQz9RGoSHhkyMgqEBdNPFWSWMY+1nBPxy+MjvZ2aZxB9n/zz3FwKiOTZfotb3AhhF" +
            "xSUUNmGSjX+vWvPPYacVWJOkUilUT05ymEVb0JFHj9l/AVn+35b/jsx6YzNz8mja+iAEH7rYDntY" +
            "Gaz3dizW080KWaeICx77kiG7lTKG6EEoPb0Wu0lZ9OA5whFH8GxHQjOMQls5HSs5t/glHX2FYtT/" +
            "mGAs/fCtFU0vQJUSQYfvIBvVyukuLhbjuood/H6WCbD/AQSFvIO3JDxgAAAAAElFTkSuQmCC"
  },
  "Yahoo": {
    image: "data:image/png;base64," +
           "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAIAAAAlC+aJAAAJjUlEQVR4Xs1aa3BU5Rl+nu/sZnPZ" +
           "sLvZAMVECgoFokUUBbnkAjNqa6W2XlqsWAE1Ee0P+8NO2z+1Y2dsx5nO9I5aBERxrDo6Vp2OZQrE" +
           "hEvRqqOFqVy8IRZCdjeBkOxmz/eUTLM7Q4BzTkOn3Wf257tnvue9PPt871lms1mUHATh6fsPtv+q" +
           "TxBBj1CDEoSQPa4dG4/B7/QAQig9iNj+XCrXHSi5pgSbh2L7oynBF1KpEZAI2n27eg6+PQh/kCVX" +
           "AVrK2bI6Y6wB5FcsWKq0ZoAyPV25t17oY4DMEqi/PFRiLUR1buh2TzDIrAhobo2XEAFJyJuONT2C" +
           "/GNhKsbm59yYNCWkPsC7m1Ld+60J1D/u3GXxSKUpFQIUCWx+JEMQQRBCyx21ggxKBl0fZHdvGghW" +
           "LkxbXPG5KRGSJUOA2LImbXJCMLS0JsQS8UIChMF+27khLQTSn/gkM/OaGGEBhATQKxoUwMBHARQs" +
           "uPhYAYD++kKq/wgI+pIlsGBlzCkjYACENv/uaLGIxecVTWwoiqZbE4ADBlXCreuPugMQzAhmxTRQ" +
           "EnXJtdVjJ5YDAEGh/bEMZYgA2Sq3TctrIIAEEPrDfV0gIBRpnAJjJ19SVT+zggiEA2/2bVx1xFjH" +
           "wwhYMhLPL7w1CQgghQ/fPvHhzqyBA9BXamd9rXLM+DIJLNhpQqdEnQLX6Xiie+nD9WAgG9yxLuNY" +
           "B4DHURzpyltikWozfCpiy+9TcA38QdA235WkoEILhuBXs50be2960IYr6BeIbK9949kewfGMk3Vs" +
           "c2uSAghBJ9LY9cyxgHM2YaYzbX61KAa/D/R3880/piH/yB3Pp3O9Xs0DSOCUpvK66RXicP63b+x2" +
           "j9EEIqDGO2tgQBBgUAIAO9f2iNanOy061mYowROWWtQaK6oN89i6phvBEIpr/tLkyCEVXV/Xuvf1" +
           "/iP7ct5G4KN3T3z8Ro6ghxQKjNXp0utqUAja/Xpv1+6gEn3l0jGRMSN12tTNDAt+GDQdT6QhSDqb" +
           "De5YmzIu4YeFK+NO2AxLrrD50Qxkg/gfOWpprQEo6hQCzW0JerYtIcDseCrt5l2ejWCfdg4Nog9M" +
           "xF14exLDYObT7HsvH2cwrzplYVldQ+XpFTZXfiMZqc3DCyRs70G881rP6VohUcKuF9PZNDybR4Iu" +
           "XlKVrIsUxsluXZtGLti1lra5NcEzJyXKebcl/ESABuhc22t5hvqIbufQ+HoVkjCiPdkD4rBu53Ps" +
           "XJ9iMDMWncDLliTOKLVGxKK7kjY8CB9wz2t9mYO502frsz3Z/dsG4OkCBDu+ITSjsZoQAEhvvpI+" +
           "9kmQm7sELFwed8pwxmBDceyF5Q1XVwrysYxZs31jCiNAdK5LG9fQr4ZNQxIugAAFtD+aBhTAftKU" +
           "uY0ras8WaERRWrxqKMKzByCwY31a7ilB+QFt39gjvyyGY+6CWxIFkjq0p3/v1uzwU32gi64dUzMx" +
           "RAE8YwVAkA2Lq8dNc7zzQSC9D3vaeymhgL+9nBroIrxgBTNnaaw8YQoSrpPmx7GGgL+AEs1tMcCA" +
           "BHjW1aIxbGwbQ/988KRdKzYVpI7H05LPj5eMbbkzUfQvA73a+XSvgAAf1X6BF7VUA/Jf7i741tiX" +
           "ftQ92OvdzXrnpWPHj+ajtWFCR/Zn923NAd4+j5Pnh+q/WAkSgKRdL3YPDshUWO/+EUGoqS3pLVSF" +
           "9wOC6D5136cdq09Q3nqiG36euPqecQCff+Cfm37a42sBbn9y7LybasACSYvgd7zhQMK/ApRpaR3b" +
           "8dgHypNed0a77fFj16wan3e5Y0PG9woSnWBnL0mIJAowUuEqU/AFpAR6sPJdrxMg62aUT22O0HOO" +
           "Ceez9wb37up7+0+pY4coz9MTZv7yeDhCjsiqQAnD5ydhRWJUMCM8w6JVNdZX28iT2t+5pgfyVl66" +
           "4XzzilqSwyMvQLavO/fnX3f94voPf9Dw/v0X/OPBy99f13bo769lYAlJECQExinvyATZPH948e6e" +
           "D7zu14LCVXSzUp7wxMVfLbvn2ckEIQmU3L880v3SA125jITiYlkAXdrJ8yO3rz7vvKnlIgiOpgIA" +
           "HAdNrQnBCwQH+6zN+1g3O6QhCQoAREruhu988tx3u3OZEBEyEIZBAUb8qDP3s8UHPnqrjxp1C4Eg" +
           "mm6rDUddPw6G3hyh2ummoSUOCgClVx4+vP3xAYqERihL8Rct12V+882Dfen8aAgUUV0bnn1zFUCM" +
           "FgQA03hH3HEoAMLhA/2vPpSC6F1YgL0fD1EVBGiUBEAtunsc6GL0kBN1Fy5LgoAMgE2/Sak/RPg+" +
           "U4La1/W4/YQ4SgICJs2qnDwvIgijxRU3VVfVOIUOxzsvH1exNr4DljYH3uodfQUISmq+N0GY0XFw" +
           "TWHzA1Doy9j0wbwJRIAEKR7eNzj6GQBA8vLraqrOzwP8zzfNmjSnfNJl0cJ32d+bM9YgOCCbA3gO" +
           "BAA45WpcWTOqZblpao2RAgFARFV1xNIiOMDoeCOcGwECzSuSKHcthOAAK8a5c29IQiiiMs5kfTh4" +
           "Aa0ZnHxpFOdIQDBjJpTN/nqUACAExrxvx0Ll5pR+MJh5fTRYCUSYCxsrknVlAM+JACAjtdydBIXA" +
           "UDjfsjLBkYR11b3JcIVVAAmCca/7fq0g6lxbiCCnzInWzw4rGAkBDVdVjrugErRF2gQJ1k6KLPlJ" +
           "jShBOlvvQAIa761qWBQDgw+xJ0i0tCUAAS78oKHlfQ0owYxoAAJX3zP+qu9FYexZyNNSc1ZGlj40" +
           "EURwGPhh7s2x8vGgT6QEJCabmdfEz2hiRcLoxh+f3/ZsXe10WAyXovBhfIq7Yv345b+dxDAgIjBC" +
           "8AZtqDxcNyO8/7CXdRZIoOnOGI2XOwIx6yuxmV+K7d12/P3O4+lPcgDi9eEvzK+eOj8aKmMx+r9H" +
           "QObw3r59r2cJA3hSiNrGZbWkz5/cSDoOpjVVzWisUrGqHL1r8fnTn6Bn7v90yy99dsiCrlhWuXLN" +
           "RIL438LAEwO97vYnewj5qndza2H5WlIEtj2TyqUc35PVX+pcMGcMpRIiIEgW7aszvlbCAk1tcQOV" +
           "VgUo7GnPHNqdJ+h9+oqknXtjshBVQi3EzaszxvoskAnMvTVWXu1ABEuphY5+nH3v1RO+C2Q5+Za7" +
           "aiCCKJUWEgBpy5pu5UhPtydoekvlhKkRUUDJtBClwQF1rs1ABLzfb/Pk+AoGKCUCIHY8l+o/Yvxe" +
           "eyHxecz6coIAwBIiYMX2R9K+SSW4YOitNUAQ/zf8C4sZTcVG5HPrAAAAAElFTkSuQmCC"
  },
  "Bing": {
    image: "data:image/png;base64," +
           "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAMAAACdt4HsAAACslBMVEX+sgf9vwj+uQj+wwn+uQn+" +
           "wwj/rgf+xAn+sgb+rgb9wAj9wwn9wwj9vgj+uAj+swf9wAlQUFD+wAn+xAj+ughQUE/+wQn9wglQ" +
           "T1D9vAj9sgf9sQf+sQdPUE/9uwj+twj9vwn9wQn/uQn9ugj9uQj+rgf+vAlPT1D9sgb9wQj+rQb9" +
           "vQj+wAj+sAf+rwb9uAhPUFD9tAf9swf+ugn+rQf+tQf9vgn9sQb+tAf+wgn+vQn+uwj9twf9tgf+" +
           "sAb9vQn+sQZQT09RUE/+wghPT0/+uwn+vAj9xAj9uwl8aj39tQf9wgj+uAlTUk7EkR//rgZ8bD10" +
           "ZED+tghnXUZXVEz+vghVUk1cV0r6qwjuuQ+ugij+tQiogir9rQf+vwjlpBGHbzj7sAhbVUtkW0do" +
           "XUX7rAhzY0HhohJdWUp8bT3anRZdWEp8aD18aT1dV0r2rQrKnh23iiT8vwnRmRnyuQ3ZqBeBazv3" +
           "uwvqpw/6sAi2iSXSmRneoBSkhi3wqgxmXEbcnxSNcja+lyKVfDOdey/rpw5eWEn+vQjzqwt8az3o" +
           "pg/+vgm7jCK2iST8sQfFkh5rX0RiW0iCbDr3rgl9aTxoYEblrhKthChvZEL+twdyZ0GGczn+tgfn" +
           "pRCQdDRSUU9+ajzYnBaOczX0vA2agTFUUk5iWkiNdTf6tgrztg2khy1YVUyjfyz2rQmdeS/9sAf3" +
           "sAquiCmDbDqvgyiogSq4iCRiWkfJlB29iyLCjSBpXUVXVE34rgn+uAd/bTyEbDmbeDBxYkGIdDno" +
           "qxG0hiVrYUTOoRzRlRlkWkdZVEz9uQfPlxrzpwuMcDaNcTbspA7goROqgirAjCDfoBNvYULTlhic" +
           "ey/pog98Zz2zhSZZVUxVUk6SczTvpQ30rAvdnBRtYEPYmRf9tgj9ugf+rwf/rQfADIlEAAAE5ElE" +
           "QVR4Xo3XY7fkShgF4GKIpo1j22ds29a1bdu2bdu2bf+PW0ljJp0J9rdeq/dTb1VqpVcDrpMU9ZAo" +
           "zwedAlVU/uYiMob5SjD2CkDYjow6Qk2juFZP9HkFav1+CSvV/gW9AHgDIEyQRQwg6BJMy22K1V6Q" +
           "9AhAOMdYHyVGaXV4uEBgbW8AhFm9T/o7aChUBjomWN0rAOE81kdkqDr93MJNAHgHIAwXixpqivGV" +
           "RDWj6BWAaj8pktkFWSmvPrPESt4BCFWkIbIFV45Oyfpahl2AYyW/+fqQMTlUeXIJn15xBvzLbjzt" +
           "uKOl2vUh2ajMG5HbewFwB4LLArOmX3XOkSf4YTCfQOE8VvTdK/LqQZDyCOiJrLrlbHZ9xuTy8njL" +
           "oPHgvAMsh2Rnx6i++xDumGyxNt0BP6Z6nUZfA/Z90GYLHGicvVxoAI4RHQG6sAQEZ6DLAcB43Afc" +
           "krYH9HvjHq3PDMC9hzgCvISUhhCpARBKB1WBwwWQOmIYpF2AYjhWCKMKAGGrvC8Atl191HmCGyDx" +
           "eAwRHZBaFV4xAacEAjOWf/VNSnACoIypP4yiPI7xPKYmILU8o1srvz/mJAeArcvPbIpSHvOUFi7a" +
           "FxCWGEAmHl9795LfBZstQGy8pvmQjBeWkoftC4DtuwLVZNYcf9mJ+wWkVqowA+NR9r4BZgCcnAnU" +
           "Eg/M2LPp1JRlC1DClAFyPgsEUA9cc74B1IxZkTO2mgDEzkCWeSq3N8gyHEnWA+DZeKAuB5sAqMZo" +
           "CGN1sBuHZDpTswCnr3IGFiqUKrFu0NIxNyTzIbp6fh0gnHnW4kDcHqAKjTb1l4DQwTOAxXQPyrn4" +
           "3J97bAGpMBURhHxCB7UDWC684f4eG2B2GKpNiOSANhSTQzaAsP3SndNtABSW9N+fRF8OoamjWNkP" +
           "kLz8iisztlsoMoAFfpwrEg2xC20BUp8sj0QCrgDUGFBcRMarL5T4Lh0QknddG2HPwA1ggjqvQdPf" +
           "LPMr/euu94Hh5JfbIgFzMosP2C8gxSAEvhwhZaD55vW09dbbbr8jUJfInUbfCvhjrf4pothVvonN" +
           "r9+D6fp778uY2/GenUtSgh2A4RSO5VB9mQceJA89/Ii5zdg1jz7GKnYAX+isAJHHn3jyqbUBy/RP" +
           "bx0GToBSBjKRZ3Y/93xzc127eeULKQAcAd4n6sDiF196uad+9VmvbHoVABcglG3kuIE33nxrunX6" +
           "t9/xCUk3gJdSHCe+O8Paf+99KMFEt88NCDaInDjwwYfm7cc/2rAuPdEOWdobFoAWJwB2sk1wn34W" +
           "2Qv0RD7/QuSWip0r5hmGWppMOgDdIgPEga931IBvv2tr4/SI6Y0o3KTqSHbCZwfALn0ErvGHH+PG" +
           "8jt+2q3PVBa6wGA36p+jQj/MJ7p708kWCxCEk+XlBjZv+IX9LP76m/GxRkwDvTmkkfHqgbQIFkAV" +
           "K4WBP/7c85fImdIoTgGgL4eKCIUTKszDodKkGWDCYFUQ121u4+rTJqaB4FuhIYKIYeTrgWC7uJRz" +
           "ChuC5W/N+AuCxqdaANjnDLCznAbAtORGjegEsQIJ0RkwzlLPSE6fwgpA0OgqdBmEMJIjFoAJ/4ic" +
           "W9oqQwDfCisQzKcbXQWRnWUlVgA26CO4byNtAzDhXwZ42YYdIE38xwCPQ1gBFtjFAI9D/A+btEBN" +
           "Oyr4fwAAAABJRU5ErkJggg=="
  },
  "Ecosia": {
    image: "data:image/png;base64," +
           "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAYAAACqaXHeAAAN9klEQVR4XuVbCVQUV9Z+1SvVQAPN" +
           "urQgiAiouCcY3BOjOCExjkuCGDWa/DHbjOPEjEuMSeY3GZ2YxTgmkkTNoAbJuBJ1EgUNUXFwRdki" +
           "myzNYjcNNN1N71PfOXlnQIVuZmjhHMrTx+9offd+71a9V7fufcXYbDbSnw9ebzit17fJDpdXz+y3" +
           "AfjgSsHqRadzvr2t0Qb2uwCYrFb+7uLy51vNZo+P84p/1+8CcE2pHtZkNPkCZ9xWPNEd7tGKmsdH" +
           "pZ+8wNmI7nMBuHKncdifL+e/oWwzeHRFqNBoQyku07RGOHjX8JadubjtqZPZJ6+pmuJUbUafPheA" +
           "W82tEW/l3tgc/e3xgpx65ejOCAz3h2KrzTH/L57N3fZ1UfmroIPny4rv9LkADPF0/wWAuwOCnjyR" +
           "fUxtMLrfjzDIw62EYh8X+wM5UlE9g1szVlCOr4tYEeMlLe5zARjh7VnwiL9PJvAdLggphaXP348Q" +
           "y503wFVSCjzax+uyPbsfXS9eSa88jleGDf5UwOPZ+lwAGIYhBx5/JClW5pELwrk6Zfz9CHzuvLWj" +
           "Y94DThocurcrm2arlTlfr5xIucNkHpdXj4zeSnrwEPSksWBXSX3ub2eMT71VMV9vtkg6ndMxg/bo" +
           "zGZxcuTAA13ZazWZWZPVJgGWu7KlR2ZOnM0K+Kae1OxoKoyrQbgzGSFuvwd0WKw2xnv3QUViaFDG" +
           "lvEj/xQgYVX2OFzg+UIeY8U06dEAnFU0xE09mpnNLUA3lkcP2snNxZ1cMKyOcG1mE89Y9HOc8UbW" +
           "VFPppbFmxS+RVnVtkM2gcwOBEUtaeV6BCkFQ5C/CQWMviYZPzRJFTchhBEKH7Ku4R+97l/PXppdW" +
           "zVPo9AN/eGLKlOnygJ96NAAWm40ZnnbiSmFTy0gQpgX5HT82a9IciUBg6HQO15aEaI9ve6Ute98i" +
           "a4uyW2kvT+pT6zIx6e+us17bLgiMqOyMUKPV+U04fDqbyy8iwRvs4Zafv2DWCO7iWHo0AADXVero" +
           "R49lneYSkUAQXoqJ+HjHpLEr7wmWqsZPs2/du/qzqc8Tq0XY0SNj5vuGlvF9QioZibQFBJuuRWpR" +
           "VoZY7twOJzaboGMk+CZ2cvLX7kn/v4HvHdxwt69nT53f9W1J5RLgUDfJre9nTf7NUJnHLYcHhQB0" +
           "51ep0QYszczZ4f31PxTinWktJoulA0f745cLa5OkKsXTxEZ/tYtkdU2fr/hIf/n7aRZtk6Qz2/g/" +
           "nINzwWlvAzZh+26O766DFeGpRwvXXby+rtlglHR3PP9TPUBvNvNYgcAKbDO2CZv/9sJ2/U+pL1AC" +
           "z1te5jZv/buSKc/tY0RsN1Zv2NMLdWe+SWpN//MGq6o6nBLYSckpHi+nvMKIXExOfgo4Dqx6jUS9" +
           "KfGgMf/sDIKD4Zlcn1r1vvuCjZsYscTQboUW/Vx3J+5SQ+O4spbW8EaD0Qt8mVikDpe6lY31k+VO" +
           "CPDNYQV8I+VwC6VYk7ZxrfbIh2uIzSqEHdHQyf/0WntsDo911/V6AHDlG9+beYwOnnH3rvNalTZX" +
           "HPvoOUrQthUPP1Z+4bXP8jXz6vSMZ6nWG67va89NIGiaEy4/8PvYIZ+M8vEqoDYMeafj1R8u+M6m" +
           "UQXQIMjeOpmIO6FXA9D08aKd9LbnyYLKvd/JfFQQPKScVn4Kar/4q9SatphhCA8EuEv8eQmpbZPa" +
           "TQOSIkJ3b40ftdqfdWkE11xTHKZ6e9ppa6MijE4Hz9///cVeS4V1p75aiMHTK08HT/OG2AMnrrfp" +
           "M5Zi8G0WPils8SVv5iU4MngA/r6S28tgA7ZAgG34gC8Q4BsaeiUAeNS17PrDp3TO47angy9UXk2Y" +
           "8X1WZoPeIE+viiWrryeQiZkrSPLFZ8nphsEd7Ih5JjLNr4SM9arixNyb98DG4xlnMrkaYgINAnzB" +
           "J/jQAC0PfAo0bVvyuT5rz/8Bu85+413pc5vfBq7VFMRV1L2aueraDPaCKrRLG8M9asmWEceJr1hL" +
           "cCgNEpJ9J4ybHu7EU9hGAlw0JIDVEH+xhrACRj/Af/u0UI8hOeC2fLP6He3hLRuA2amLv/B8bfdL" +
           "D+xlCBkekhz6qMNqD6w1tsgOF29PP1U7ni3XenXKnx2UT3LVcrJk4GU6eBzER6wjT8vzO6Oxt+vX" +
           "pXu47B7hKXZrhE999v5kPCKhxW3u+k3IGB/IFEB6SzM8POfpo+7Fn/I3v3xpovxgzXBS18U8Twgs" +
           "IunjU8kk37Ju+XXjK+Q3FCmbgeETvoGhBZqcvgbQFxvk9sCMm6weSQ7wVaU6hlu0ltjjB7PNxFus" +
           "I2K+hfCY7gvmmw4tqWypiQGGb2gAhiZoc3oA8FZHX2zY+AX7aYaHEjf02eOvicoiYa5qUsFNkUPV" +
           "Q4nJ2j0JLnwzv1r9HXwR+IYGYGiCNqcHAK+0FIvHPXGMZngHy6rnO3L1R3opAEmoRE0KNX5k7vlk" +
           "ckwRjQKpw4eh7cf5bWaziGpor83pAcD7PH2rE0XF5wAiveUaHZ72Bv/FmIOE5Zt/pRPyJ+5uWMwt" +
           "hDeaA8hfiqY4rMFD0OhZ21oUBwwN0EK1OT0AKGYA4JWWJ/HQAV9XNoyzx3sz6gwJZDUdnTOEzJHn" +
           "k7XRWWRN9Jlu6TAYC+CTQAO0UG1ODwAqOQB4n6eEJ/x/CH8r5hRhuyjXydnm7r9n2FCKY+4fAJMi" +
           "HIBqodqcHgBaxkIx4z9Km2Wzgwu4K5nZKa9a79G9NNssJCuvJZKFXPZosd0bBCFplAFQLVRbr5TF" +
           "eYwNgCQEFJN58rz7prSfl8YRJEdGB1b9ghY/knTxGZKtDCMlrT7kUqOc4PimYhR5++ZjpJULjpBn" +
           "YXqlL4ACJi1jUQKfJ1UTurBxc3n/+H0kRKK+a1D+3Iq/iEzKXIGBdOljT8UYUqXzIvQ4WTeEVGg9" +
           "yae3JpCM2hiyq3wcEfLdmkCgWqg2pwcA1VsA1PAoQSgILu3Q/nJtJDKR7r58k41Ptt2KJyoDe1/7" +
           "p+ojyNmG8A6cDEUUefXKbGL7tYZQ1iojIkFQGTDVQrU5PQAoXQOggGnVNUuAJeKoS+0JuY1ycq0p" +
           "uPOFlHNbrpXdY/tyYzBZkzcTQbr7/A71g0FuKuIqjsoFhgZoodqcHgDU7QFQvTUWnYsD9JIMv8Aw" +
           "Li3AzSYxSSl/yK4dmUh/j+2M2igM1i63XDeg2Y0deR4YGqCFanN6ANC0oNiQm5EIwOeJjB6S+CPN" +
           "RheyMOdZckUtt2snRNJ0V+eJIXlNgYTlG+1zPePS+DyxgWpor83pAUDHBk0LYP25tGdRvQX2kSam" +
           "vFf4qMXBag/RmoUdBr8xfzoZ7K4kC0Ou2qNaUC+klWNooI0UaHN6ANCuQscG2Nba6I/SNbBU8lB2" +
           "oHTabkft5KoHUEj2Vo4iJ+qiiM4sIodqhnXJQ52QFkvhGxqAoQnaHkgegHYVOjbAqNujdA2MAqYf" +
           "K662bwF5wcOkWiclx2uHkB0l4wmOc6qBRGV07ZQD2/BBy+XwTbtH0NQnSmIoYKKGZ7RaWdKDh4jH" +
           "03NNz2mTg/wcKYk5PxNEr45hpY3AaFqgbg8MgWnTH/ktBPfk4GGTDh6+4BMYGqDlgVeF0aiULt36" +
           "OjA6NmhaoG4Pwuww+QlcLfvTwT6ADdiCTdobgC/aJYIGaOmVvoDksWV70ZwARscGTQsIpHdC3vyE" +
           "Edyi9RVWbtL9wwIubMBW+8YIfNHGCDT06h4hNCrRpgJGx0a5Nv48nQ7o6Ox9bPzyK3NnxD4XOXAn" +
           "2l727OEcnAsOuLBBb3vYhg/aGoPvzuw4rT0ObDBb+Gdr6uNSi8vngWPRtUiU6yefpK1sxRyesXnP" +
           "G+9Y27Ti9jydySz6oap20qbL+auWZ13cPudk9j78gPFv+D+c054DG7AFm9Q+fMEnWvMpBSWLDpVV" +
           "zVK3GVz/m7F0m/BVYWmS/+5DlWTHflts2vF/UY7VoBeqP0re2b6nX7dcXtr6z88XWw06YXf9gAMu" +
           "bLS3CR/wRTlBew6XQYv7l+mqXYWlzzg1AB9eK1zBObPCIf7+9tbtp8DpzQ0Se4rK5rfXtOPmrcXO" +
           "2CCBTq/XwNSjVW0Wq6uAYYybHo794xsjo7eB0JtbZAC23fhl+R/OX/3MbLOJ3YQCdVXykyGeYlFr" +
           "jyZCOfXKUe9fKVwzVCbNfy4yLDXKS1pKCb25SYqCAnVzxJeFZcuK1C0xG8cN2/CQn/d1J2WCji+u" +
           "DMM4cZucfdArAeCuQPiyM/9KmRkSePKtMUO39JTd9NLKJzdeurlxZeyQvy6LCt+H4Pa1DybwLUDw" +
           "5KOnf+L2906zWK1CB3afMh9cLXjdaLHy7eYbImFzgbpl1Atnc/dyv0/65BcjS7MuftmgNwQDR3ra" +
           "39K+Na/4tTUX8z7Zcq1wpT3bU4L8fx7gJikD/qqo7PWM2zXT+1QALtQpR59RNMz8NZtrTggJ/KEr" +
           "gsFi4W++WrgG+G/5JS/bm4oiPs+yfcKYlzjBZnC4zZHP9KUAoD84keINY4eu9xKLNF0RztcpH1YZ" +
           "jAHACp0+rFqrt7vNJXFg8I9pj8c/7eMiVlS16uR9KgAakxkruu314ZEfrBoR9Zk9wo3G5tiOW+NN" +
           "no74mRs+ION2cmL41kdGrepT3wtMDPTNzkiYNP03oUGnHdzhLWvPZwUCjaO+sEF7jK/sZp8KgKPb" +
           "0ynwZcXKdiu8klvg6vrNl6MAU4P8szBlgJ8Ok3/HZxhbvwoAtrTPHBBwhOXzNetGx7zfr74dpmDH" +
           "pHGv7Hts/IIID/fK3tLQ7z+f/zdtBQxrg2hCXQAAAABJRU5ErkJggg=="
  },
  "Ekoru": {
    image: "data:image/png;base64," +
           "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAMAAACdt4HsAAACplBMVEVCrUIApdYYrbUhrbVzvUJ1" +
           "wUaBxj8Apc4AnNpHskcApd7O7/f///8praRovUc3s4dCtYRKuXuMyDxuwFWHyU1WvWtnv2lUvXUI" +
           "pcYAnM4InM4Ipc4Qpb0QrcYQrb0IpdZKtUoYrb1dwNuUzjmczjmUzkKl3ufG7+9St0TO9/fn9/f3" +
           "//8trZRjwWMYpa0InMYhraQQpcZzvTl7vTlGuIdKtVJSuVIMnNpLuGmMzkJauVZhwFghrb0ttb0Y" +
           "n9Mcps4mq881scpBtsI8sNZHt9BOvc4xsKJoxdNrxt5zxt6E0uK11jE8sp95w1Jctz+czkKUzkqM" +
           "zoyl1qXG58aM1udjvT2p4uu95+9avUrG7/fO7+9rxmd/x2Te7/f39/d4xXCDyYeMztYApcbv9+/W" +
           "79YQpa9GtT0rq9ilzkqUzlKUzmsIrcat2JGczpyt2Jyt2qW93q233re93r3O58pSvWOIyliU1t6U" +
           "1uec1udSrUpMut613ue15+dcv8zG5+fG5+8ArecIpd4QpdoIstt3zNfW587n7+fe79Z3zORjtUoQ" +
           "nM4Qpc7v9/f3//elzjmUxkKc2toYra1SvUoYpd4Ard6t5++15+8YpcbW7+/e78a33qet1jnO571S" +
           "tTne797v9+ec0I6MzpQIlM4Ypb2t1lKg3uJrvTkhta2UzlqUznuczmMprb295+ex1mcIrc4InL2U" +
           "zpQYnN7O5++l1oiE0taUzpwAlNYQrc6czkrW9/etzjGt1jGt27re9/e93ojn9+cArdYAnOcKqrpf" +
           "v46957XL57e13sa93sal1mul1jkApefW786t1krW796UzozW7/e11kqcznu11jlrxka11lLn79a9" +
           "584hnL3G584AlN6t1kLG797v9/8htb2lzmeM1tb///eg1q3q6+s+AAAJxklEQVR4XoWXY5NsSxqF" +
           "K7dVtG00bds2jm3bNi5t2zbGtuefzJt7V+OeqohZ1dHR/WE9+SpRqmi2RDGeQ+JkNIfuzwJM5rZj" +
           "gpiLcQ9AXLKL45GRzsDhxsbG5sOBzoNHZWwOhCrX6mI00tlSf8WLOI5D8OG83vrmwIGoiBFZgKzc" +
           "pyKBDVewFSGWZZEiIDkaOsZEOYjcgEnFHh1pOY29SFCcCoITBGRH3J1ABCNyApTlowdanEjgBMQi" +
           "zs7abC6QzWZjZRJEJZwOjOOGZAOUGo0dvgzL2WFR27q8VEpn3Quq0OtT6nX5GCAAecNBcSVBtdI/" +
           "Uq9E7XSpdebqiYlYjAepeD42sVendkFFcGKnO6IrCqFa9k9dLBfwGm/k6dLVfdjrKy0DlZYe4lXr" +
           "fVXmtnVKPbiW+5cJquX1O8oF3DeXtlrTx6t8VQvbdg+vBZ0b3jU4X+Xj+TJ9ygW1AW1YJqiW/J0O" +
           "XCR7gZme4PnShd1rpJVaM/yUBRC6PMQh+GmJxsUMINM/8QBeX3C2pak+3rew9j9SluaG53lfhdqG" +
           "7E6OC0xlCKpMAPfV49a5eoxUTGXZc03KqWvDab5M68IxnOqCqZcBygBED0PvkT0Fft+bXyxbrs4e" +
           "P378wURyEXH+TV9VyToBCPVjSjNVsl/sOi0ILCqopmJlQ3OL5lffba19pQb0z6azRxKZPHZXlenX" +
           "4UI0K2VQyQmMb8Aj8oyZjvGL/tl3m0iCIEmSAPn9/qazJ5JKGmW39S6EkPeinIRK7sBFL/jzS4yX" +
           "fE8p/qv7a8FHYj9J1PgxwV/TPiWj/3KoqoQAQsN9cQxYCgAVGGnVvNK8qWNuD4mlxnrhBUABw1O7" +
           "/yom7C6d0NmgZwERCCo5gFMnEXJZoQBrZf/RJsJPki+0legtM6B5i7VETfoJwuM/NosJQ2XVKQdC" +
           "dWMZQFQOIGXU8Ntk/6u1hJ8ge8wzA/3910OgYKhyxtpDEh6Pv/VBXIfB2N48xJ7CIWDAAe8ZhBxW" +
           "+lLZeew/3gR29TcmbO0PZdT/4T4rROF3N2LC4+kJnROdrB+XAWJAuItQXjXFL8j1a4Xwe6YHwGsw" +
           "MHRxMcMY+kPB/tANS5vf4/G04oZ+OmHO4wRvVzwOgGjDSQF5tcyobw8GrAK/dh8OnKF7eze/9NJM" +
           "L00xBpyJUe0miMJ2vDUsE1qnHbWIGBA5DQCb2dCXxi18sJYgtTPXP+5nqN6tF9bgjNec295LMTeA" +
           "kC7wEO7CI0DYdcnsgjKOY0C7AD0gjTdigxLoLFHTNh0MhRjqnSekJT2xvbfIEA5XWkmP3/1KAodQ" +
           "XYDQqREAiC1QAlZNF/NDYEg0+knrQChEax5OSit1YYYyGMImnZvwuFbhVlZpEfoqAIBo3ckziNXS" +
           "Gt85MBzxE21QAAP1zr078onfUpX9wTTpcRc+ACGsrdbbWKEZunD/5ZNfIqeOnjh0AWdAkiUDoTDV" +
           "C/Hfo/MzVGXQpAOA7SbkkDa72K/qTqjiEQeUwKmnYqWPQw+bCPU0BKDZLmXrW01xMGy55XazLTDS" +
           "C1V5SHCMq+IjXrkJRT48RrO1ZA9kwPQCDAQdOH9+bulY20wZwjMp9we2B2Cadkyk4M47qop3obsA" +
           "2EsdkgGvECXhUKhoZk62X9j6k5c3P7kbyLL20IzBpHW73baDkjQ8oeW48ucw4AwCQFHMB6u+Td6y" +
           "hkPXi7bKhu+efeSRjc9ufHHhT5kQoAo3rB4gBJLSeYvOznn34xQwwAyAtQoAZpjaiQ0P/eYXr33+" +
           "2saNv/tw/pcK4clRU6UFAB4owpppHYu8EMH+8kXALgD4MSBIP4oNn//hoWvS3M9f6+7+MHPObB9l" +
           "DBhgg1mam9evFjBgrPxnGED14UGarSH0YZgCGfDQH5UR2vjJT+d/kAE7MgD2gVnp2rzeJnjH8CAJ" +
           "AgbE+HlcRA+MQZD6buV98K/uT/Y9Jc/lTg1jmJYBcLr9qsImOKGNkxswwEpN8JaklGzy40EsfvhH" +
           "AMjhm7kVgELWOS5JgxU2dBkA8VbhLuvQ9VLrcR9XeQgLbP3Ncytm+MXuj//Wk5BT6GMGrMTXhTbX" +
           "CRkgNEwCQD6SUxpNDJ8HJ2r8WlOwEqq4pJ3B7u5NPfJxuhVqUOJxnXHiSVqocAot4uJ5UFD9+1js" +
           "PUlKtnsKpoNBw74LS1vgxf7r3ZZjSXkORhmmZ0vhGbZ+FnfBIcBuzJypLjOlWW+BHThVS2hNoWDl" +
           "vm+VLC68HLp+fVPJTcx63EiZjAWsm2UbEnAi6GCOMEAMINgVOtr0mLyhb9aQ1k0QA7P54Uf37Hxn" +
           "IAx+a1NCko+hYpOZeL3wdbYVJjGtFa4cxQCYRSSwKSMcim9CCFfP+tUWaGU4bDDduAFnU2jA2nZE" +
           "bsdCH2PSuWwfOFe3J6Vze9XchkkAyDkglGehNHwV3jaJdpkQDikKz1jbVslTsNZIGYwFhZ4tbH5E" +
           "kr7f+2euIy4DxA4OZlFP0zEcAo6hhiyZGcDmgU1wo9Sukltw7UkNw1hdq1ezuIb/eK+i8PQYBoAi" +
           "dzh8MdCXeOVkTx5p8qv1lulpuNPayKbnkpISAG0wpraAH5dgdkHLNYsZgBjgOHyqUbHF2zXxbm0N" +
           "3I5wM9eums3s5QUNY9K7PBgQkZJ/t+aV74+LMAdKCAKHnkkXaXjfECQhI179d3vrsbORxOLzZhuM" +
           "cXUBuwX89Qnps+/1TvmJAQAlBMSilIa+zVftzv082nGJMdBaG8t+ybIXpavif1N1kbgCwBqrRxyb" +
           "r9XQPlXZnlz+oZiRYfRfy8/3hkTyo+PfPNOFn7yLALHTLnCCUwc7gq8azvKvGYxRxcXmPCQDDkqf" +
           "xafaR0SwKW8krGizwLHoDT1NXeJLt93zyNxliT0P/lssbiFqTCZgbRH7lWeeEsJYPYcEzqU30hBE" +
           "escXS+5zQ5ZYn7GYtuax7GpYv/7tRFxRBjCZIXQ5IQmhUGekqNHH+KrBoR9Anw5afHxMw9BGPfhx" +
           "/JcPXv1o2a+8VDOEDq8gQAwps+b55zW3fbwPxPPrY30UXZzWusBtRw5HV2LZv/KxDYoGyjmEkBNe" +
           "+xoNVUSNjvaNauCvYli+wMauRo6n3y/v/LW46J/MAFYSHPi1JTjztPo0TVG0IrO2wAnR2x3vOy53" +
           "isv+rC8cOIsrwldQCc7uIlPaCrPZatZr1eucCNntDke+o2Eky48BKwlddWDn7Ahxdnt+vi0/34EU" +
           "99OgxgjY7/ED4MeESEs553DYQQ744F8OB2De/+vTdR3j8Sx/1tc+KERXcx3orbfuLAv+bzh2n5jL" +
           "D4B7CdCesciBrs6OTvxzsbOza+TA2Pjksj33F08ljf+nbH/0f+C6zXUwzYSLAAAAAElFTkSuQmCC"
  },
  "generic": { // generic search image to be used if no engine logo is present
    image: "data:image/png;base64," +
           "iVBORw0KGgoAAAANSUhEUgAAAEAAAABACAYAAACqaXHeAAATM0lEQVR4XuWbeXRUVbbGv3PvrUqq" +
           "UhkqA2FIGBQZA2HQfoKi4IA0Dg2IQ4s2iIqNgsrQyCCjIA0oo0AghKYRCCKTMgYEZEqQKQ1CGAIk" +
           "IYGEIfNYVffe/fZddXHV48Far/utssX+1vp5bpn8Ud93zt5nV0Xxa5QQQmIpsiwrEotf3/5zyCzz" +
           "5zIgBH4DEoYZ5k5mJCbQxIo7SGYZwd2bzoWQfXc50Ga/r2PHTn8aOvQvCxOXLN3//fe7Mw8eTCtI" +
           "S/vx2sGDqblbt6UcnzP3y3Vvv/PuqJZxrR4FEHBbEOKesS6x4JWlZVz8S3+dNiPl5E+n1KtXrtKJ" +
           "9H/QhnXr6IsZn1dPHDe+eOyYMcWTJkwo/3LuXNq+dRudO3OW8vMLKDX10KVBgz+cGhlVqzFMGaXx" +
           "a9/1n896o/se6LkwIfEf16/foGNHjtKoj0fmtmvbbmeQPSgJwDRmLDPcZCTzKTM/3Bm+ocvjnU/N" +
           "njlTvXThImVnX64ZOWrMHD5B0bfK4ld75M2nyD/17b8qj3f7+LFj9MpLLx8JsAYsAPAR05t5lGkG" +
           "iHoQgVGAEg4ggqnLNGceZ15nJtaKitoyfOjQkit5eXQ8/cSVhzt07A2WecLEr858kCOk7cxZczMr" +
           "Kypp3CdjM9n4PABvMp3C6zSJfey5/gFDJiy0JK7eHfrtzvQGO1IzW2354dSD67bufXj+kuT4vgOG" +
           "xDSLa28DhAVADPMkM6xBbP1dq1eupIrySnqz/ztTboUgWL8a86Fh4R1XJX9TfK2ggJ7o8sQOAB8y" +
           "nds90qP2iC82i6OZ1VY3UQMieoj5PdOHeZ8Zw8xgFjPL3Sol7Np3bNDr/Qc3sdpCBYB6TFdm1rCP" +
           "Pip0uVw0ctQnST7lIP6tNQ+W3e6IX/b3lcVXcnMprmXcagCv1Ylt1mzE51uUzacIu7NInL5BASU1" +
           "al1N9bRTVbWzR9W6q6r2KvMuvx7h8ahTVNWTQEQbmcPMsfRTF2c83+v1GAAWIdDaKKM3Xnstx1Xj" +
           "osEfDJn/b22Mwivuy4pzzNiJF4puFlKb+DbJAHq0/l33ektSCrHzAontZ1Vpb44qDhToOFJMuOwi" +
           "m4cojoieYXowfZkPmHHMF7pOSRzKWg5jHxFdYc4tSFzRI8AWAgE0BtD/zwMG5FZVVlHXZ34/2CeE" +
           "f0/dP9PtueSS4hLq8YceWw3zjz87MHrtUcKWUyTtOK+KtAISx0v4BFSTOK8yOodAJKqJGhPRs8zL" +
           "zNvMMOZTZh6zgtmqadoBDiKbiNSUPUeG20OjwWrE/Hn+vHllWVk5NVFR0a2FEGDkX9x8reg6z6Wm" +
           "HabFCYvOA+jX8sHn6yenkvjupGr5/oKmHLpGyokyUs64Sbmok5JDpPCWWq7ymk8EDuF+IurFvOHT" +
           "D6YzicxaZjdz3O325BDrh0MZQ2o3aAdWk7DQsGlnTp+mWbPnHgQgGYIpAf9LAEJ+7/0P0j8eMax5" +
           "mzatZ8iB0V9PSTqaWbuuXbLZgZBQiYw1MBCwKoAFgMJIjDnogwC3A2hqBeIB2JkQJpQJM3Ey4Uwo" +
           "l0WkoshRe44X9nm11wvbr+ekPtD7xRdnLlqc2LFDx0dePX/uzNdGKei6rkl+3n0FAEVE1nq+X7++" +
           "cYmLFx4tLi5Je2fU6oLouo5AxUIBQQ4pICAQgQZWhQECLYxirjIQoDD8HOICcjQgF155TFwmNUyV" +
           "sXLDLyVdQ8sWEfcPHLusxhHerPzbb9cvyzx3Ths0aNBoM18d5oMfRTpYj3Z6bGB0VATmz//yhyd7" +
           "jslp16GNpGuaPcgpU0AAYGBVGACKicyYE4wJBAEBbuBiIFBbADafENy3giCgQghEV6jywvRcbe7D" +
           "HR8I6dl/ivrVF/1OJSUtTh09dkKnTz4Z83hZackuozwVf3Z+ItIVi7Ve9+7dO6em7s/T5IhTvfuN" +
           "rNFUPSg4TILVCljZvMVqmoa5mggf4A1EIsCjcggWoJa5i5pPEFUCiHQRtl+qwXSHXQqPi6BpFX/o" +
           "vuhYWp+Cnbt37x43TnTq0vmJN/hE7OIygD9LQGbgDI/sHN8qzsKf4n7q1mPANWeUwyrLcFgDhJ2N" +
           "2xULIxjALvsgGMlEMCaBEhCqAdU6cN3MSTNxMVYPcCRfYD7JEM0ixYehgXgprnHg4526/lGDJexi" +
           "Rsapqq5duz4DwMFoCvys2JiYDg6HHWXlNZc6Pt3TraoUbA+CsFhAigIosnncb9t13wFemPi81FUO" +
           "wArUBxBgBmBVgbPFwHc64Glow2uhAr0JqKzrRJdmLeI3n3vgd6Vnz57PbNOufTwgWnATPOzPAHQG" +
           "9erFtFA9LkREx16vHdvY4tJEgMUiSJYBA0li/ne9/yzpLtcKAaoOlErem8CiAZcqgEM6QJHA0yEC" +
           "LxJQKiBkhxX1o6ND6jZq+pDrZlFWbmRkZHxIaGhz7gN+C0AQkW6stWvXqauqmrt+oybV1kDF5nKR" +
           "RZIFSZJpXvgE4GNYmNxFQgCaDlQa+alATg1wgQAlFOhk4wAAVAtAIkK1JMgRHiJiasc0zRGleaVW" +
           "qwWhoaENOQD4uwSsFqslRJalmrr1YomAYAmkSEIQhNeFr8Q/OZzogOQBLruBEgDBxpxgAeLNW0Hx" +
           "6Q0OCYgKi6h9jdRQN0iHwxEcIYTwewCSpmmSlVt9rVo2pYjgAIRCAOH/L5DXoSazOSvQUALqAvCY" +
           "5s0fQzXQCXZHUHCQ7HTKRAQhpEBe/R6Ap6a6pkqxWMKiIu2OkkJ4NIKi6yAiQGd8RT6I/1sAJAOK" +
           "FYi51dUZKyOZAegmbiJYA23WYIceEsKbgqqqqprw8HC/BUCMYKmFhTcLQWjkDLE5rWVQtXJvALoG" +
           "EKMTIxgf8/BZ71YS5FMGBAQIQIbPFG1i5ihcbhWSPUAKjQgMC6uoqERlRXlpeLgT/hyEJCLS8vPz" +
           "s8orKh+MbVAvMtiG6jwNVtUDUjVAu4UC6PAi+ZgTvs93OSk6QAQownsdWm4fJIWA5NbgKquEHGHX" +
           "nBG2kNp5eXkoLLpZcOPGNb+WgGBw48b1o1nZl3u3b9+6XmQolYKE1e0CcQjwMKpqBMCY71r3uQ3M" +
           "Z9P0XcuFNMAmAYGmeYuJQgRJCMg3y1DGgQeEBakOZ3Bw3e0pp926puVZLFa/BkAMiouL9p84eVL0" +
           "6vFswzrhuBoUCEt1NcjhNgLwYrEAsszcZQagO5wA+JwAj7cEAs0MFWEGoPNq4fViPtUE20VQWJA7" +
           "UlaUkAOpqWcA5AB+bIJEpAEQrpqq48ePp2fnXcmvH1s/tlZMFHnO5QqppgawBpjvkJElRtyWoE8h" +
           "053rHxpAvCrkbX4ymSEYqxCwVKjApXygZSOEhdmoQXb2VRxKTT0qhLgGluLvL0Pcbrfr3Nkzq/bt" +
           "Tx39ep9XYprEUGFWvlDKy0E/B6B4AxDW/7m78m27L24zT4zKsCQjBEYiL7KmQgqQYPnpIqqtFmGP" +
           "iagJDtSssSkpO12FN6/vlySpTFVVv1+DOoObNwqWbN2eMrT777vG1A4PoyaxhJ8uCtgCAUUGfP+8" +
           "SRZGAIpPUxR3aYK6GQCZbUMnCF1j8x4IaJAKKyEys0l7pLUIdciukOy8awHJq5P3CiGOMzW6rvs3" +
           "ACLSjVNQVlaadezI4SXr1n876J23+0W1aqi7C4qEKCr27r6QfGpaZ6yAJt3hPjPRb5tyNB0gDULz" +
           "eFHdEMbrU2ehNawjlPrOakl3i7AVK1a5zp/L2CDLch6E0AAIBX4WsQCIrKzMyStXJb/aocN/Rca1" +
           "bO55uIUudh+XUFjodUfE6LeuRp/SMEdm6fbdJ69xVfU2Uo1h4wZCN+bjPCDQCnqoiQYLgF2pP8pL" +
           "lyZtEkAa73wREYFFAv4XOHGFj5saHBL2x2e6Pfu3hIXzLOHOMCm7UMf+ExLcGsBDGWxBjA2wBjJW" +
           "wKIAEiObJSJM80RmUCqgegCXC/Awqse7Xr8O2KxAl7Y6nFYPnb+QLd56e8DFtNR9owHsYwpuu2n8" +
           "2wg1TVOJCNWV5Zs8bteNseMmivLyCr1hhITO/CaDbd43XVIElJUB5aUm5UAlU3H7WualnCkz19IS" +
           "Y+YA8q4AzmDgifYaIm26fiX/hhg3flIpm5/PG3GSx/Ib8JHwo3Fx62Nx27Zt7+MvQ21PPdU1of87" +
           "Ax798VCanpmZKU2eNIGczjBRXK0h/YKMS1e9W+JwmCchwFsGkuRFmKWim6WiqoDbBZ7rwaMtoEhA" +
           "84ZAfEMNNousX8rKlsaOm1C5ZvWqL3gPNgshTgGo5hLwbwASi+uMANCUz6a+N3rUyGn8d3zrhawc" +
           "a+P7GlGd2rXE9M/n0EkekCaMG4PGje/TCZAuXddx9rLAtSIBlbx9IMBqzgnmlAQdUDVv3bvd3hKw" +
           "KgDPF2hen1DPKREA/fCRY/Knkz8rTdm2ebbH405h82eJqBj+lqIoMljBwcHyipUr5xHr7Lnz9Pms" +
           "ObRuw0atsqpazzibWfPkU09P556w6LkXXtTXrN1A/KlRIyLd+EfOTY0OnNVoQ5pOX+0mStpBlLjd" +
           "JIVoKb9esYdo02GdDmdqlF+qEUtntOKiElq0eCm1afvQOUmShwLowOYj8EtIYYHVomXL6MOHj+wg" +
           "1vYdO9V33xus/3nQhzR12gztxE8Zeq/evZMAPCwEou1BjndbtIy/MPD9D2nn93uosrLSMKIxuvFQ" +
           "Uk2UW6TRhWsanS/Q6CKvV0t0KnPRz6YNiouLacPGTfTSK69XRURGrwPwRwG0ZfOh8LfMY6+A1a1b" +
           "t/aXc/MyibVwUaLntTfepH5vDaCPR4+jlJ27tGvXb1Dbtu0+5t8P5aYkzHbRNswZPqdVfPusvv3e" +
           "ocSkv1N6+gkqKSnxMXk7Ot28cZPSDh2m2XMXUI+eL1fExDTcJcnyMABd2HgTALZfwrhgZLD6vdn/" +
           "Ra79cmM3Jk7+zNN/wEAa/OFQGjBwEK1M/lotLC6hYcOGbQTwNAdgNZslTDn58TGbLWh83Xr1d7Z/" +
           "sMOVHr1e8Qz+aDhNmTqD5s5LoAULE2kOm5346VR6d+Bg6tb9hZJmzVudCA51rhBCGgzgSaaleeSl" +
           "X8S8sTCYNn36cGKdO59JI0Z9og39y0gaO34SvcUhzJ47XyvgnZ82fdpeAC/wzt9/2xsUhgBYmDCm" +
           "GdNNlpWBHMhn9qDgRWHOyOTwiFprHcGhywNtQfMUxTIWwFtMN+ZB5n42Hu5/4yZsQgIg+OtlrP76" +
           "69nEOvTjYW34iNH6pMlT6a8zZtL7HwyhcRMna9k5lyk5efVJAC+z+Wbs1YK7yOfP1zYzjDrMfUxT" +
           "piXTgmli/rsYY7eZQPyLEv+qeU3TdO70lvUbNq546sknXk7hZvfDvgNyndrRgusQeTyRGL/zp9f7" +
           "SDdvXs994fnnx3ODOwLA6A+uf2KewB2+NdcNiHUHP+TvnZfBahkXF7J//4EdxFq3fqN7/KQptHTZ" +
           "V8xymjx1Gg0Z/rG+d/9B4wosadig4RAA8VwyNv9ukP+vOQms1q3jI3IuXz5ArNVff+OZ9vks+mbd" +
           "Blr9zTqaPW8+DRsxSt+0ZZt+Nb/A1a5d+08BPMS5hcD/8r/5Vq1aR7H548akt3xlsmfeggTalrKD" +
           "2DAtWbqMRn0yTk9es1Yr4o7fs1fP+QAe4eCicC/LbHiIi2sVkZWVnW6YT+adT+Ljvu/AQdrzwz7j" +
           "NU3gMli2fIVWXFJKAwa8uwpAFzZfB4C4583zdBfCV9xBYq1dv9GzavUaOpb+Dzp89Bh9+90WnvI+" +
           "p8Sly7QiHl4mTZq0GcAzbD4WgHTPm2/cuHHw6YwzB4i18bvNnvUbv+PmlkkZZ87Rju9308zZX9Ki" +
           "xCT1Gk9ms2bP3g3geTbfULBwr0piARB2u922Zeu2ncTavHW7Z8u2FMrNu0I5uXm0/2AazU9YTIuX" +
           "LFXzC65RQkJCKoCeHNwD9+x/u+874dnsdvmbteu2EGv3nr3unbv2UGFRMd1kjh5LN5oez+1/U41A" +
           "Fi1anAbgJc6tqTnI3Jsyj61kDwriCW/NGmLtP3DQs2fvfqqqrqEal5t+Op1By1esoqS//V3Nysml" +
           "hQmLbpk3pjzlnjbPJuTg4BAsX7EyyRxvPQf4qHs8HjJ0PvMCreKOz92ezV+mBQsWHjLNN7+HzZuY" +
           "R3f2nLkziXX0eLrnYNqPpKoaGcrKzjEGHvpqVbJ24eIlmjlr1v57fud9m57FYsHQYcNH60SUkXHW" +
           "k+pjnj/jGzeAsftq5oVLNG3a9B8A9DDNW+5582A1bda8bVV1NfGXiSofe93jUYlldH3jBqA1a9dr" +
           "59n8xImTtgP4A3f7pve6eQk+qqioKK+oqCwrLS0T7dq2gaLIYPM4dToDLrdbb9UqTkpakrh5/Phx" +
           "X7L5DF3XLxKRB78Fmd/q4NnnnutLXqkXL2Xp/DWWvuG7TfqpjDP03vvvrwXQnX/3fvOq+22JM5DA" +
           "6tu376e5eVdpN8/4mzZvNa496tOnz1cAusqK0giAhN+ohPlFJdo/+NAovupqfjxy1NWzZ68EAE8q" +
           "itLA94PNbz4Ep9P5Ymxs7AgAXTiYGAAy/kMkzHHYUD1+jgIg8J8m8//Sxn+6BH7D+m9uoBi14hOM" +
           "NwAAAABJRU5ErkJggg=="
  }
};
//@line 8 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\search.js"

// This global tracks if the page has been set up before, to prevent double inits
var gInitialized = false;
var gObserver = new MutationObserver(function (mutations) {
  for (let mutation of mutations) {
    if (mutation.attributeName == "searchEngineURL") {
      setupSearchEngine();
      if (!gInitialized) {
        gInitialized = true;
      }
      return;
    }
  }
});

window.addEventListener("pageshow", function () {
  window.gObserver.observe(document.documentElement, { attributes: true });
});

window.addEventListener("pagehide", function() {
  window.gObserver.disconnect();
});

function onSearchSubmit(aEvent) {
  let searchTerms = document.getElementById("searchText").value;
  let searchURL = document.documentElement.getAttribute("searchEngineURL");

  if (searchURL && searchTerms.length > 0) {
    const SEARCH_TOKEN = "_searchTerms_";
    let searchPostData = document.documentElement.getAttribute("searchEnginePostData");
    if (searchPostData) {
      // Check if a post form already exists. If so, remove it.
      const POST_FORM_NAME = "searchFormPost";
      let form = document.forms[POST_FORM_NAME];
      if (form) {
        form.parentNode.removeChild(form);
      }

      // Create a new post form.
      form = document.body.appendChild(document.createElement("form"));
      form.setAttribute("name", POST_FORM_NAME);
      // Set the URL to submit the form to.
      form.setAttribute("action", searchURL.replace(SEARCH_TOKEN, searchTerms));
      form.setAttribute("method", "post");

      // Create new <input type=hidden> elements for search param.
      searchPostData = searchPostData.split("&");
      for (let postVar of searchPostData) {
        let [name, value] = postVar.split("=");
        if (value == SEARCH_TOKEN) {
          value = searchTerms;
        }
        let input = document.createElement("input");
        input.setAttribute("type", "hidden");
        input.setAttribute("name", name);
        input.setAttribute("value", value);
        form.appendChild(input);
      }
      // Submit the form.
      form.submit();
    } else {
      searchURL = searchURL.replace(SEARCH_TOKEN, encodeURIComponent(searchTerms));
      window.location.href = searchURL;
    }
  }

  aEvent.preventDefault();
}


function setupSearchEngine() {
  let searchText = document.getElementById("searchText");
  let searchEngineName = document.documentElement.getAttribute("searchEngineName");
  let searchEngineInfo = SEARCH_ENGINES[searchEngineName];
  let logoElt = document.getElementById("searchEngineLogo");

  // Add search engine logo.
  if (searchEngineInfo && searchEngineInfo.image) {
    logoElt.parentNode.hidden = false;
    logoElt.src = searchEngineInfo.image;
    logoElt.alt = searchEngineName;
    searchText.placeholder = "";
  } else {
    logoElt.parentNode.hidden = false;
    logoElt.src = SEARCH_ENGINES['generic'].image;
    searchText.placeholder = searchEngineName;
  }
}
//@line 67 "c:\binoczilla\src\binoc-central\ux\browser\components\newtab\newTab.js"

// Everything is loaded. Initialize the New Tab Page.
gPage.init();
